from pydantic import BaseModel, StrictStr, StrictInt
from .cloudwatch_metric import CloudWatchMetric
from .cloudwatch_statistics import CloudWatchStatistics
from .cloudwatch_unit import CloudWatchUnit


class CloudWatchMetricStat(BaseModel):
    Metric: CloudWatchMetric
    Period: StrictInt
    Stat: CloudWatchStatistics | StrictStr
    Unit: CloudWatchUnit | None = None



from pydantic import BaseModel, StrictStr, Field
from .match_option_types import MatchOptionTypes


class CostCategory(BaseModel):
    Key: StrictStr = Field(
        min_length=1,
        max_length=50,
    )
    MatchOptions: list[MatchOptionTypes]
    Values: list[StrictStr] = Field(pattern=r'[\S\s]*')

    def dump(self):
        return self.model_dump()



from pydantic import BaseModel, StrictStr
from .elasticache_endpoint import ElasticacheEndpoint


class ElasticacheNodeGroupMember(BaseModel):
    CacheClusterId: StrictStr
    CacheNodeId: StrictStr
    ReadEndpoint: ElasticacheEndpoint
    PreferredAvailabilityZone: StrictStr
    PreferredOutpostArn: StrictStr
    CurrentRole: StrictStr


from typing import Literal, Optional
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)


class AWSs3VersioningConfiguration(AWSBoto3Base):
    mfa_delete: Optional[
        Literal['Enabled', 'Disabled']
    ]=None
    status: Optional[
        Literal['Enabled', 'Suspended']
    ]=None

    def to_options(self):

        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'mfa'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'mfa_delete'
                ]
            )
        }


        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items()
        }

        parsed_options.update(uppercased_options)

        return parsed_options



from typing import Literal


MatchOptionTypes = Literal[
    "EQUALS",
    "ABSENT",
    "STARTS_WITH",
    "ENDS_WITH",
    "CONTAINS",
    "CASE_SENSITIVE",
    "CASE_INSENSITIVE",
    "GREATER_THAN_OR_EQUAL",
]



import asyncio
import functools
import boto3
from botocore.config import Config
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.exceptions import UnsetAWSConnectionException
from werkflow_aws.models import (
    AWSCredentialsSet,
    AWSRegionMap,
    RegionName,
)
from werkflow_aws.models.ec2 import (
    EC2DescribeVolumesQuery,
    EC2DescribeVolumesResponse,
)
from werkflow_aws.types import EC2Client
from werkflow_aws.helpers import collect_paginator
from werkflow_system import System


class AWSEC2:
    
    def __init__(self) -> None:
        
        self._system = System()

        self._loop: asyncio.AbstractEventLoop | None = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None

        self.service_name = 'EC2'
        self._regions = AWSRegionMap()

    async def sso(
        self,
        profile_name: str,
    ):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        await self._loop.run_in_executor(
            None,
            functools.partial(
                boto3.setup_default_session,
                profile_name=profile_name
            )
        )

        self._client: EC2Client = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'ec2',
            )
        )

    async def connect(
        self,
        credentials: AWSCredentialsSet,
        region: RegionName,
    ):
        
        aws_region = self._regions.get(region)

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        self._client: EC2Client = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                boto3.client,
                'elasticache',
                aws_access_key_id=credentials.aws_access_key_id,
                aws_secret_access_key=credentials.aws_secret_access_key,
                aws_session_token=credentials.aws_session_token,
                config=Config(
                    region_name=aws_region.value
                )
            )
        )

    async def describe_results(
        self,
        query: EC2DescribeVolumesQuery
    ):
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name,
            )
        
        dumped = query.model_dump(exclude_none=True)
            
        result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.describe_volumes,
                **dumped
            )
        )

        return EC2DescribeVolumesResponse(**result)

    async def describe_results_paginated(
        self,
        query: EC2DescribeVolumesQuery
    ) -> list[EC2DescribeVolumesResponse]:
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client is None:
            raise UnsetAWSConnectionException(
                self.service_name,
            )
        
        dumped = query.model_dump(exclude_none=True)

        paginator = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                self._client.get_paginator,
                'describe_volumes',
            )
        )

        pagination_result = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                paginator.paginate,
                **dumped
            )
        )

        results = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                collect_paginator,
                pagination_result,
            )
        )

        return [
            EC2DescribeVolumesResponse(**result['Volumes']) for result in results
        ]
    
    async def close(self):

        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        if self._client:
            await self._loop.run_in_executor(
                None,
                self._client.close
            )
            
        await self._system.close()
        self._executor.shutdown()

    def abort(self):
        self._client.close()
        self._system.abort()
        self._executor.shutdown()


from pydantic import BaseModel, StrictStr
from .cloudwatch_dimension import CloudWatchDimension


class CloudWatchMetric(BaseModel):
    MetricName: StrictStr
    Namespace: StrictStr
    Dimensions: list[CloudWatchDimension] | None = None



import asyncio
import configparser
import functools
import pathlib
from concurrent.futures import ThreadPoolExecutor
from werkflow_system import System
from werkflow_aws.models.parsing import AWSConfigRole


class AWSConfigParser:

    def __init__(self):
        self._loop: asyncio.AbstractEventLoop | None = None
        self._system = System()
        self._parser = configparser.ConfigParser()

        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._matching_roles: dict[str, AWSConfigRole] = {}
        self._roles: list[AWSConfigRole] = []
        self._active_role_idx = 0

    def __iter__(self):
        for role in self._matching_roles.values():
            yield role

    def __next__(self):

        if len(self._roles) < 1:
            raise StopIteration('No roles found! Did you call load() first?')
        
        role = self._roles[self._active_role_idx]
        self._active_role_idx = (self._active_role_idx + 1) % len(self._matching_roles)
        return role

    def __getitem__(self, role_name: str):
        return self._matching_roles[role_name]
    
    def first(self):
        if len(self._roles) < 1:
            raise StopIteration('No roles found! Did you call load() first?')

        return self._roles[0]

    def next(self):
        return self.__next__()
    
    def get(self, role_name: str):
        return self._matching_roles.get(role_name)

    async def load(
        self,
        path: str,
        account: str,
    ):
        if self._loop is None:
            self._loop = asyncio.get_event_loop()

        path: pathlib.Path = await self._loop.run_in_executor(
            self._executor,
            functools.partial(
                pathlib.Path,
                path
            )
        )

        absolute_path = await self._loop.run_in_executor(
            self._executor,
            path.absolute
        )

        resolved_path = await self._loop.run_in_executor(
            self._executor,
            absolute_path.resolve
        )

        await self._loop.run_in_executor(
            self._executor,
            self._parser.read,
            str(resolved_path)
        )

        self._matching_roles.clear()
        self._roles.clear()
        
        for section in self._parser.sections():
            section_line: str = section
            if section_line.startswith(f"profile {account}"):
                role_name = section_line.split()[1]  # Extract the role name from the section header

                role = AWSConfigRole(
                    role_name=role_name,
                    region=self._parser.get(section, 'region', fallback='N/A'),
                    source_profile=self._parser.get(section, 'source_profile', fallback='N/A'),
                    external_id=self._parser.get(section, 'external_id', fallback='N/A'),
                    role_arn=self._parser.get(section, 'role_arn', fallback='N/A'),
                )

                self._roles.append(role)
                self._matching_roles[role_name] = role

        return list(self._roles)
    
    async def close(self):
        await self._system.close()

    def abort(self):
        self._system.abort()


from pydantic import BaseModel, StrictStr
from typing import Literal

EC2Filters = Literal[
    "attachment.attach-time",
    "attachment.delete-on-termination",
    "attachment.device",
    "attachment.instance-id",
    "attachment.status",
    "availability-zone",
    "create-time",
    "encrypted",
    "fast-restored",
    "multi-attach-enabled",
    "operator.managed",
    "operator.principal",
    "size",
    "snapshot-id",
    "status",
    "tag",
    "tag-key",
    "volume-id",
    "volume-type"
]


class EC2Filter(BaseModel):
    Name: EC2Filters
    Value: list[StrictStr]


import datetime
from abc import ABC, abstractmethod
from typing import Literal, Any

CloudWatchMetricDimension = dict[
    Literal['Name', 'Value'],
    str
]

CloudWatchStatistics = Literal[
    "SampleCount",
    "Average",
    "Sum",
    "Minimum",
    "Maximum"
]


CloudWatchUnit = Literal[
    "Seconds",
    "Microseconds",
    "Milliseconds",
    "Bytes",
    'Kilobytes',
    'Megabytes',
    'Gigabytes',
    'Terabytes',
    'Bits',
    'Kilobits',
    'Megabits',
    'Gigabits',
    'Terabits',
    'Percent',
    'Count',
    'Bytes/Second',
    'Kilobytes/Second',
    'Megabytes/Second',
    'Gigabytes/Second',
    'Terabytes/Second',
    'Bits/Second',
    'Kilobits/Second',
    'Megabits/Second',
    'Gigabits/Second',
    'Terabits/Second',
    'Count/Second',
    'None'
]

CloudWatchScanBy = Literal[
    "TimestampDescending",
    "TimestampAscending",
]

CloudWatchLabelOptions = dict[
    Literal["Timezone"],
    str,
]

CloudWatchMetic = dict[
    Literal[
        "Namespace",
        "MetricName",
        "Dimensions"
    ],
    str | CloudWatchMetricDimension
]

CloudWatchMetricStat = dict[
    Literal[
        "Metric",
        "Period",
        "Stat",
        "Unit"
    ],
    str | int | CloudWatchMetic | CloudWatchUnit
]

CloudWatchMetricDataQuery = dict[
    Literal[
        "Id",
        "MetricStat",
        "Expression",
        "Label",
        "ReturnData",
        "Period",
        "AccountId",
    ],
    str | bool | int | CloudWatchMetricStat
]

class CloudWatchClient(ABC):

    @abstractmethod
    def get_metric_statistics(
        self,
        Namespace: str = None,
        MetricName: str = None,
        Dimensions: list[CloudWatchMetricDimension] | None = None,
        StartTime: datetime.datetime | None = None,
        EndTime: datetime.datetime | None = None,
        Period: int | None = None,
        Statistics: list[CloudWatchStatistics] | None = None,
        ExtendedStatistics: list[str] | None = None,
        Unit: CloudWatchUnit = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def list_metrics(
        self,
        Namespace: str = None,
        MetricName: str = None,
        Dimensions: list[CloudWatchMetricDimension] | None = None,
        NextToken: str | None = None,
        RecentlyActive: Literal['PT3H'] = None,
        IncludeLinkedAccounts: bool = False,
        OwningAccount: str = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_metric_data(
        self,
        MetricDataQueries: list[CloudWatchMetricDataQuery] = None,
        StartTime: datetime.datetime = None,
        EndTime: datetime.datetime = None,
        NextToken: str | None = None,
        ScanBy: CloudWatchScanBy =None,
        MaxDatapoints: int = None,
        LabelOptions: CloudWatchLabelOptions | None = None,
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def close(self):
        pass


from enum import Enum
from typing import Literal

RegionName = Literal[
    'af-south-1',
    'ap-east-1',
    'ap-northeast-1',
    'ap-northeast-2',
    'ap-northeast-3',
    'ap-south-1',
    'ap-south-2',
    'ap-southeast-1',
    'ap-southeast-2',
    'ap-southeast-3',
    'ap-southeast-4',
    'ca-central-1',
    'eu-central-1',
    'eu-central-2',
    'eu-north-1',
    'eu-south-1',
    'eu-south-2',
    'eu-west-1',
    'eu-west-2',
    'eu-west-3',
    'il-central-1',
    'me-central-1',
    'me-south-1',
    'sa-east-1',
    'us-east-1',
    'us-east-2',
    'us-gov-east-1',
    'us-gov-west-1',
    'us-west-1',
    'us-west-2',
]


class AWSRegion(Enum):
    AF_SOUTH_1='af-south-1'
    AP_EAST_1='ap-east-1'
    AP_NORTHEAST_1='ap-northeast-1'
    AP_NORTHEAST_2='ap-northeast-2'
    AP_NORTHEAST_3='ap-northeast-3'
    AP_SOUTH_1='ap-south-1'
    AP_SOUTH_2='ap-south-2'
    AP_SOUTHEAST_1='ap-southeast-1'
    AP_SOUTHEAST_2='ap-southeast-2'
    AP_SOUTHEAST_3='ap-southeast-3'
    AP_SOUTHEAST_4='ap-southeast-4'
    CA_CENTRAL_1='ca-central-1'
    EU_CENTRAL_1='eu-central-1'
    EU_CENTRAL_2='eu-central-2'
    EU_NORTH_1='eu-north-1'
    EU_SOUTH_1='eu-south-1'
    EU_SOUTH_2='eu-south-2'
    EU_WEST_1='eu-west-1'
    EU_WEST_2='eu-west-2'
    EU_WEST_3='eu-west-3'
    IL_CENTRAL_1='il-central-1'
    ME_CENTRAL_1='me-central-1'
    ME_SOUTH_1='me-south-1'
    SA_EAST_1='sa-east-1'
    US_EAST_1='us-east-1'
    US_EAST_2='us-east-2'
    US_GOV_EAST_1='us-gov-east-1'
    US_GOV_WEST_1='us-gov-west-1'
    US_WEST_1='us-west-1'
    US_WEST_2='us-west-2'


class AWSRegionMap:

    def __init__(self):
        self._region_map = {
            'af-south-1': AWSRegion.AF_SOUTH_1,
            'ap-east-1': AWSRegion.AP_EAST_1,
            'ap-northeast-1': AWSRegion.AP_NORTHEAST_1,
            'ap-northeast-2': AWSRegion.AP_NORTHEAST_2,
            'ap-northeast-3': AWSRegion.AP_NORTHEAST_3,
            'ap-south-1': AWSRegion.AP_SOUTH_1,
            'ap-south-2': AWSRegion.AP_SOUTH_2,
            'ap-southeast-1': AWSRegion.AP_SOUTHEAST_1,
            'ap-southeast-2': AWSRegion.AP_SOUTHEAST_2,
            'ap-southeast-3': AWSRegion.AP_SOUTHEAST_3,
            'ap-southeast-4': AWSRegion.AP_SOUTHEAST_4,
            'ca-central-1': AWSRegion.CA_CENTRAL_1,
            'eu-central-1': AWSRegion.EU_CENTRAL_1,
            'eu-central-2': AWSRegion.EU_CENTRAL_2,
            'eu-north-1': AWSRegion.EU_NORTH_1,
            'eu-south-1': AWSRegion.EU_SOUTH_1,
            'eu-south-2': AWSRegion.EU_SOUTH_2,
            'eu-west-1': AWSRegion.EU_WEST_1,
            'eu-west-2': AWSRegion.EU_WEST_2,
            'eu-west-3': AWSRegion.EU_WEST_3,
            'il-central-1': AWSRegion.IL_CENTRAL_1,
            'me-central-1': AWSRegion.ME_CENTRAL_1,
            'me-south-1': AWSRegion.ME_SOUTH_1,
            'sa-east-1': AWSRegion.SA_EAST_1,
            'us-east-1': AWSRegion.US_EAST_1,
            'us-east-2': AWSRegion.US_EAST_2,
            'us-gov-east-1': AWSRegion.US_GOV_EAST_1,
            'us-gov-west-1': AWSRegion.US_GOV_WEST_1,
            'us-west-1': AWSRegion.US_WEST_1,
            'us-west-2': AWSRegion.US_WEST_2,
        }

    def get(
        self,
        region_name: RegionName,
        default_region: AWSRegion = AWSRegion.US_EAST_1
    ):
        return self._region_map.get(region_name, default_region)




import asyncio
from concurrent.futures import ThreadPoolExecutor
from werkflow_aws.models import AWSRegionMap
from werkflow_system import System


class AWSElasticBlockStorage:
    
    def __init__(self) -> None:
        
        self._system = System()

        self._loop: asyncio.AbstractEventLoop | None = None
        self._executor = ThreadPoolExecutor(
            max_workers=self._system.configuration.cores.physical
        )

        self._client = None

        self.service_name = 'ElasticBlockStorage'
        self._regions = AWSRegionMap()


from __future__ import annotations
from botocore.paginate import Paginator
from abc import ABC, abstractmethod
from typing import Literal, Any


EC2Filters = Literal[
    "attachment.attach-time",
    "attachment.delete-on-termination",
    "attachment.device",
    "attachment.instance-id",
    "attachment.status",
    "availability-zone",
    "create-time",
    "encrypted",
    "fast-restored",
    "multi-attach-enabled",
    "operator.managed",
    "operator.principal",
    "size",
    "snapshot-id",
    "status",
    "tag",
    "tag-key",
    "volume-id",
    "volume-type"
]

Filter = dict[
    Literal['Name', 'Values'],
    EC2Filters | list[str]
]



class EC2Client(ABC):

    @abstractmethod
    def describe_volumes(
        self,
        VolumeIds: list[str] | None = None,
        DryRun: bool = False,
        Filters: list[Filter] | None = None,
        NextToken: str | None = None,
        MaxResults: int | None = None
    ) -> dict[str, Any]:
        pass

    @abstractmethod
    def get_paginator(
        self,
        Operation: str,
    ) -> Paginator:
        pass

    @abstractmethod
    def close(self):
        pass


from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool,
    StrictInt
)
from .ec2_filter import EC2Filter


class EC2DescribeVolumesQuery(BaseModel):
    VolumeIds: list[StrictStr] | None = None
    DryRun: StrictBool = False
    Filters: list[EC2Filter] | None = None
    NextToken: StrictStr | None = None
    MaxResults: StrictInt | None = None


from pydantic import (
    StrictStr
)
from typing import Optional
from werkflow_aws.models.base import AWSBoto3Base


class AWSs3DeleteObjectTaggingOptions(AWSBoto3Base):
    version_id: Optional[StrictStr]=None
    expected_bucket_owner: Optional[StrictStr]=None


import datetime
from pydantic import BaseModel, StrictStr, StrictInt
from .cloudwatch_metric_data_query import CloudWatchMetricDataQuery
from .cloudwatch_scan_by import CloudWatchScanBy
from .cloudwatch_label_options import CloudWatchLabelOptions

class CloudWatchGetMetricDataRequest(BaseModel):
    MetricDataQueries: list[CloudWatchMetricDataQuery]
    StartTime: datetime.datetime
    EndTime: datetime.datetime
    NextToken: StrictStr | None = None
    ScanBy: CloudWatchScanBy | None = None
    MaxDataPoints: StrictInt | None = None
    LabelOptions: CloudWatchLabelOptions | None = None


from pydantic import (
    StrictStr
)
from typing import Optional, Literal
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)


class AWSs3PutObjectTaggingOptions(AWSBoto3Base):
    version_id: Optional[StrictStr]=None
    content_md5: Optional[StrictStr]=None
    checksum_algorithm: Optional[
        Literal[
            'CRC32',
            'CRC32C',
            'SHA1',
            'SHA256'
        ]
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None
    request_payer: Optional[
        Literal['requester']
    ]=None

    def to_options(self):

        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'md5'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'content_md5'
                ]
            )
        }

        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items() if key not in uppercased_options
        }

        parsed_options.update(uppercased_options)

        return parsed_options


from pydantic import BaseModel, conlist
from typing import List
from .aws_s3_multipart_upload_part import AWSs3MultipartUploadPart

class AWSs3MultipartUpload(BaseModel):
    parts: conlist(
        AWSs3MultipartUploadPart,
        min_length=1
    )

    def to_options(self):
        parts: List[AWSs3MultipartUploadPart] = self.parts
        return {
            'parts': [
                partto_data() for part in parts
            ]
        }


from pydantic import (
    BaseModel,
    StrictStr,
    StrictInt
)
from werkflow_aws.models.base import AWSBoto3Base
from werkflow_aws.models.parsing import (
    convert_key_to_boto3_arg, 
    convert_key_to_boto3_arg_upper_matching,
    key_contains_patterns
)
from typing import Optional


class AWSs3MultipartUploadPart(AWSBoto3Base):
    checksum_crc32: Optional[StrictStr]=None
    checksum_crc32c: Optional[StrictStr]=None
    checksum_sha1: Optional[StrictStr]=None
    checksum_sha256: Optional[StrictStr]=None
    e_tag: StrictStr
    part_number: StopIteration

    def to_options(self):
        options = self._filtered_options_to_dict()

        uppercased_options = {
            convert_key_to_boto3_arg_upper_matching(
                key,
                [
                    'crc32',
                    'crc32c',
                    'sha1',
                    'sha256',
                    'sse'
                ]
            ): value  for key, value in options.items() if key_contains_patterns(
                key,
                [
                    'content_md5',
                    'checksum_crc32',
                    'checksum_crc32c',
                    'checksum_sha1',
                    'checksum_sha256',
                    'grant_read_acp',
                    'grant_write_acp',
                    'sse_customer_algorithm',
                    'sse_customer_key',
                    'sse_kms_encryption_context'
                ]
            )
        }


        parsed_options = {
            convert_key_to_boto3_arg(
                key
            ): value for key, value in options.items()
        }

        parsed_options.update(uppercased_options)

        return parsed_options




from typing import Literal


CloudWatchUnit = Literal[
    "Seconds",
    "Microseconds",
    "Milliseconds",
    "Bytes",
    'Kilobytes',
    'Megabytes',
    'Gigabytes',
    'Terabytes',
    'Bits',
    'Kilobits',
    'Megabits',
    'Gigabits',
    'Terabits',
    'Percent',
    'Count',
    'Bytes/Second',
    'Kilobytes/Second',
    'Megabytes/Second',
    'Gigabytes/Second',
    'Terabytes/Second',
    'Bits/Second',
    'Kilobits/Second',
    'Megabits/Second',
    'Gigabits/Second',
    'Terabits/Second',
    'Count/Second',
    'None'
]


from .amount import Amount as Amount
from .cost_category import CostCategory as CostCategory
from .cost_explorer_query import CostExplorerQuery as CostExplorerQuery
from .cost_explorer_response import CostExplorerResponse as CostExplorerResponse
from .dimension_value_attribute import DimensionValueAttribute as DimensionValueAttribute
from .dimension import Dimension as Dimension
from .dump_options import DumpOptions as DumpOptions
from .expression import Expression as Expression
from .granularity import AWSGranularityLevel as AWSGranularityLevel
from .granularity import Granularity as Granularity
from .granularity import GranularityLevel as GranularityLevel
from .group_definition import GroupDefinition as GroupDefinition
from .group import Group as Group
from .metric import Metric as Metric
from .operations import Operations as Operations
from .result_group import ResultGroup as ResultGroup
from .result import Result as Result
from .tag_value import TagValue as TagValue
from .time_period import TimePeriod as TimePeriod
from .total import Total as Total


class UnsetAWSConnectionException(Exception):

    def __init__(
        self,
        service_name: str
    ) -> None:
        super().__init__(
            f'Err. - {service_name} - connection has not been created. Please call - connect() - before executing any other operations.'
        )


import datetime
from pydantic import BaseModel, StrictStr, StrictInt, StrictBool
from typing import List, Optional, Literal
from .aws_s3_common_prefix import AWSs3CommonPrefix
from .aws_s3_multipart_upload_response import AWSs3MultipartUploadResponse


class AWSs3ListMultipartUploadsResponse(BaseModel):
    Bucket: StrictStr
    KeyMarker: StrictStr
    UploadIdMarker: StrictStr
    NextKeyMarker: StrictStr
    Premix: StrictStr
    Delimiter: StrictStr
    NextUploadIdMarker: StrictStr
    MaxUploads: StrictInt
    IsTruncated: StrictBool
    Uploads: List[AWSs3MultipartUploadResponse]=[]
    CommonPrefixes: List[AWSs3CommonPrefix]=[]
    EncodingType: Optional[
        Literal['url']
    ]=None
    RequestCharged: Optional[
        Literal['requester']
    ]=None



import datetime
from pydantic import BaseModel, StrictStr
from .cloudwatch_dimension import CloudWatchDimension
from .cloudwatch_statistics import CloudWatchStatistics
from .cloudwatch_unit import CloudWatchUnit


class CloudWatchGetMetricStatisticsRequest(BaseModel):
    Namespace: StrictStr
    MetricName: StrictStr
    Dimensions: list[CloudWatchDimension] | None = None
    StartTime: datetime.datetime
    EndTime: datetime.datetime
    Period: int
    Statistics: list[CloudWatchStatistics] | None = None
    ExtendedStatistics: list[StrictStr] | None = None
    Unit: CloudWatchUnit | None = None


import datetime
from pydantic import BaseModel, StrictStr
from typing import Dict, Any, Literal


class AWSs3UploadPartCopyResponse(BaseModel):
    CopyPartResult: Dict[
        Literal['ETag', 'LastModified'],
        StrictStr | datetime.datetime
    ]
    ResponseMetadata: Dict[StrictStr, Any]


from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)


class AWSs3DeletedObject(BaseModel):
    Key: StrictStr
    VersionId: StrictStr
    DeleteMarker: StrictBool
    DeleteMarkerVersionId: StrictStr
    


from enum import Enum
from typing import Literal


ServiceName = Literal[
    "amplify",
    "api-gateway",
    "app-stream",
    "athena",
    "app-sync",
    "app-flow",
    "app-studio",
    "app-runner",
    "audit-manager",
    "backup",
    "bedrock",
    "cert-manager",
    "codeartifact",
    "cloud-trail",
    "cloud-wan",
    "cloud-formation",
    "cloudfront",
    "cloudhsm",
    "cloudsearch",
    "cloudtrail",
    "cloudwatch",
    "cloudwatch-events",
    "codebuild",
    "codecatalyst",
    "codecommit",
    "codeguru",
    "codepipeline",
    "codewhisperer",
    "cognito",
    "cognito-sync",
    "comprehend",
    "compute-optimizer",
    "config",
    "connect",
    "cost-explorer",
    "data-pipeline",
    "data-transfer",
    "datasync",
    "datazone",
    "deepracer",
    "detective",
    "device-farm",
    "devops-guru",
    "direct-connect",
    "directory-service",
    "dms",
    "documentdb",
    "dynamodb",
    "ec2",
    "ec2-other",
    "ecr",
    "ecr-public",
    "ecs",
    "ecs-kube",
    "efs",
    "elb",
    "emr",
    "elasticache",
    "elemential-mediastore",
    "eum",
    "enterprise-support",
    "fis",
    "fpm",
    "firewall",
    "fsx",
    "glacier",
    "glue",
    "guardduty",
    "healthimaging",
    "iam-analyzer",
    "iot",
    "iot-dm",
    "kendra",
    "kinesis",
    "kinesis-analytics",
    "kinesis-firehose",
    "kinesis-vs",
    "lake-formation",
    "lambda",
    "lex",
    "lightsail",
    "location-service",
    "lookout-metrics",
    "macie",
    "managed-blockchain",
    "managed-grafana",
    "memorydb",
    "migration-hub",
    "msk",
    "mq",
    "mwa",
    "neptune",
    "opensearch-severless",
    "opensearch",
    "personalize",
    "pinpoint",
    "polly",
    "q",
    "quantum-db",
    "quicksight",
    "rds",
    "redshift",
    "refund",
    "registrar",
    "rekognition",
    "resilience-hub",
    "rosa",
    "route53",
    "route53-arc",
    "s3",
    "sagemaker",
    "secrets-manager",
    "security-hub",
    "security-lake",
    "service-catalogue",
    "ses",
    "simpledb",
    "sns",
    "step-functions",
    "storage-gateway",
    "sqs",
    "sumerian",
    "sws",
    "systems-manager",
    "tax",
    "textract",
    "timestream",
    "transcribe",
    "transfer-family",
    "translate",
    "verified-permissions",
    "vpc",
    "waf",
    "workforce-id",
    "workspaces",
    "workspaces-tc",
    "xray",
]


class AWSServices(Enum):
    AMPLIFY='AWS Amplify'
    API_GATEWAY="Amazon API Gateway"
    APP_STREAM="Amazon AppStream"
    ATHENA="Amazon Athena"
    APP_SYNC="AWS AppSync"
    APP_FLOW="Amazon AppFlow"
    APP_STUDIO="AWS App Studio"
    APP_RUNNER="AWS App Runner"
    AUDIT_MANAGER="AWS Audit Manager"
    BACKUP="AWS Backup"
    BEDROCK="Amazon Bedrock"
    CERT_MANAGER="AWS Certificate Manager"
    CODEARTIFACT="AWS CodeArtifact"
    CLOUD_TRAIL="AWS CloudTrail"
    CLOUD_WAN="AWS Cloud WAN"
    CLOUD_FORMATION="AWS CloudFormation"
    CLOUDFRONT="Amazon CloudFront"
    CLOUDHSM="AWS CloudHSM"
    CLOUDSEARCH="Amazon CloudSearch"
    CLOUDTRAIL="AWS CloudTrail"
    CLOUDWATCH="AmazonCloudWatch"
    CLOUDWATCH_EVENTS="CloudWatch Events"
    CODEBUILD="CodeBuild"
    CODECATALYST="CodeCatalyst"
    CODECOMMIT="AWS CodeCommit"
    CODEGURU="CodeGuru"
    CODEPIPELINE="AWS CodePipeline"
    CODEWHISPERER="Amazon CodeWhisperer"
    COGNITO="Amazon Cognito"
    COGNITO_SYNC="Amazon Cognito Sync"
    COMPREHEND="Amazon Comprehend"
    COMPUTE_OPTIMIZER="AWS Compute Optimizer"
    CONFIG="AWS Config"
    CONNECT="Amazon Connect"
    COST_EXPLORER="AWS Cost Explorer"
    DATA_PIPELINE="AWS Data Pipeline"
    DATA_TRANSFER="AWS Data Transfer"
    DATASYNC="AWS DataSync"
    DATAZONE="Amazon DataZone"
    DEEPRACER="AWS DeepRacer"
    DETECTIVE="Amazon Detective"
    DEVICE_FARM="AWS Device Farm"
    DEVOPS_GURU="Amazon DevOps Guru"
    DIRECT_CONNECT="AWS Direct Connect"
    DIRECTORY_SERVICE="AWS Directory Service"
    DMS="AWS Database Migration Service"
    DOCUMENTDB="Amazon DocumentDB (with MongoDB compatibility)"
    DYNAMODB="Amazon DynamoDB"
    EC2='Amazon Elastic Compute Cloud - Compute'
    EC2_OTHER='EC2 - Other'
    ECR='Amazon EC2 Container Registry (ECR)'
    ECR_PUBLIC='Amazon Elastic Container Registry Public'
    ECS='Amazon Elastic Container Service'
    ECS_KUBE='Amazon Elastic Container Service for Kubernetes'
    EFS='Amazon Elastic File System'
    ELB='Amazon Elastic Load Balancing'
    EMR='Amazon Elastic MapReduce'
    ELASTICACHE='Amazon ElastiCache'
    ELEMENTAL_MEDIASTORE='AWS Elemental MediaStore'
    EUM='AWS End User Messaging'
    ENTERPRISE_SUPPORT='AWS Support (Enterprise)'
    FIS='AWS Fault Injection Simulator'
    FPM='FinOps Practice Management'
    FIREWALL='AWS Firewall Manager'
    FSX='Amazon FSx'
    GLACIER='Amazon Glacier'
    GLUE='AWS Glue'
    GUARDDUTY='Amazon GuardDuty'
    HEALTHIMAGING='AWS HealthImaging'
    IAM_ANALYZER='AWS Identity and Access Management Access Analyzer'
    IOT='AWS IoT'
    IOT_DM='AWS IoT Device Management'
    KENDRA='Amazon Kendra'
    KINESIS='Amazon Kinesis'
    KINESIS_ANALYTICS='Amazon Kinesis Analytics'
    KINESIS_FIREHOSE='Amazon Kinesis Firehose'
    KINESIS_VS='Amazon Kinesis Video Streams'
    LAKE_FORMATION='AWS Lake Formation'
    LAMBDA='AWS Lambda'
    LEX='Amazon Lex'
    LIGHTSAIL='Amazon Lightsail'
    LOCATION_SERVICE='Amazon Location Service'
    LOOKOUT_METRICS='Amazon Lookout for Metrics'
    MACIE='Amazon Macie'
    MANAGED_BLOCKCHAIN='Amazon Managed Blockchain'
    MANAGED_GRAFANA='Amazon Managed Grafana'
    MEMORYDB='Amazon MemoryDB'
    MIGRATION_HUB='AWS Migration Hub Refactor Spaces'
    MSK='Amazon Managed Streaming for Apache Kafka'
    MQ='Amazon MQ'
    MWA='Amazon Managed Workflows for Apache Airflow'
    NEPTUNE='Amazon Neptune'
    OPENSEARCH_SEVERLESS='Amazon OpenSearch Serverless'
    OPENSEARCH='Amazon OpenSearch Service'
    PERSONALIZE='Amazon Personalize'
    PINPOINT='Amazon Pinpoint'
    POLLY='Amazon Polly'
    Q='Amazon Q'
    QUANTUM_DB='Amazon Quantum Ledger Database'
    QUICKSIGHT='Amazon QuickSight'
    RDS='Amazon Relational Database Service'
    REDSHIFT='Amazon Redshift'
    REFUND='Refund'
    REGISTRAR='Amazon Registrar'
    REKOGNITION='Amazon Rekognition'
    RESILIENCE_HUB='AWS Resilience Hub'
    ROSA='Red Hat OpenShift Service on AWS'
    ROUTE53='Amazon Route 53'
    ROUTE53_ARC='AWS Route 53 Application Recovery Controller'
    S3='Amazon Simple Storage Service'
    SAGEMAKER='Amazon SageMaker'
    SECRETS_MANAGER='AWS Secrets Manager'
    SECURITY_HUB='AWS Security Hub'
    SECURITY_LAKE='Amazon Security Lake'
    SERVICE_CATALOGUE='AWS Service Catalog'
    SES='Amazon Simple Email Service'
    SIMPLEDB='Amazon SimpleDB'
    SNS='Amazon Simple Notification Service'
    STEP_FUNCTIONS='AWS Step Functions'
    STORAGE_GATEWAY='AWS Storage Gateway'
    SQS='Amazon Simple Queue Service'
    SUMERIAN='Amazon Sumerian'
    SWS='Amazon Simple Workflow Service'
    SYSTEMS_MANAGER='AWS Systems Manager'
    TAX='Tax'
    TEXTRACT='Amazon Textract'
    TIMESTREAM='Amazon Timestream'
    TRANSCRIBE='Amazon Transcribe'
    TRANSFER_FAMILY='AWS Transfer Family'
    TRANSLATE='Amazon Translate'
    VERIFIED_PERMISSIONS='Amazon Verified Permissions'
    VPC='Amazon Virtual Private Cloud'
    WAF='AWS WAF'
    WORKFORCE_ID='Workforce Identity Solutions'
    WORKSPACES='Amazon WorkSpaces'
    WORKSPACES_TC='Amazon WorkSpaces Thin Client'
    XRAY='AWS X-Ray'


class AWSServicesMap:
    
    def __init__(self):
        self._services_map: dict[
            ServiceName,
            str,
        ] = {
            service.name.lower().replace(
                '_',
                '-'
            ): service.value for service in AWSServices
        }

    def get(self, service_name: ServiceName):
        return self._services_map[service_name]


from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool,
    StrictInt
)
from typing import List, Literal, Optional
from .aws_s3_common_prefix import AWSs3CommonPrefix
from .aws_s3_delete_marker import AWSs3DeleteMarker
from .aws_s3_object_version import AWSs3ObjectVersion


class AWSs3ListObjectVersionsResponse(BaseModel):
    IsTruncated: StrictBool
    KeyMarker: StrictStr
    VersionIdMarker: StrictStr
    NextKeyMarker: StrictStr
    NextVersionIdMarker: StrictStr
    Versions: List[AWSs3ObjectVersion]=[]
    DeleteMarkers: List[AWSs3DeleteMarker]=[]
    Name: StrictStr
    Prefix: StrictStr
    Delimiter: StrictStr
    MaxKeys: StrictInt
    CommonPrefixes: List[AWSs3CommonPrefix]=[]
    EncodingType: Optional[
        Literal['url']
    ]=None
    ReqeustCharged: Optional[
        Literal['requester']
    ]=None


from typing import TypedDict, Optional


class s3MultipartUploadPart(TypedDict):
    ETag: str
    ChecksumCRC32: Optional[str]
    ChecksumCRC32C: Optional[str]
    ChecksumSHA1: Optional[str]
    ChecksumSHA256: Optional[str]
    PartNumber: int


from .cloudwatch_client import CloudWatchClient as CloudWatchClient
from .codeartifact_client import CodeArtifactClient as CodeArtifactClient
from .codeartifact_format import CodeArtifactFormat as CodeArtifactFormat
from .cost_explorer_client import CostExplorerClient as CostExplorerClient
from .ec2_client import EC2Client as EC2Client
from .elasticache_client import ElastiCacheClient as ElastiCacheClient
from .s3_client import s3Client as s3Client
from .sts_client import STSClient as STSClient


from pydantic import StrictStr
from werkflow_aws.models.base import AWSBoto3Base
from typing import Optional, Literal


class AWSs3AbortMultipartUploadOptions(AWSBoto3Base):
    request_payer: Optional[
        Literal['requester']
    ]=None
    expected_bucket_owner: Optional[StrictStr]=None


from __future__ import annotations
from pydantic import BaseModel
from .cost_category import CostCategory
from .dimension import Dimension
from .tag_value import TagValue

class Expression(BaseModel):
    And: list[Expression] | None = None
    CostCategories: list[CostCategory] | None = None
    Dimensions: Dimension | None = None
    Or: list[Expression] | None = None
    Not: list[Expression] | None = None
    Tags: list[TagValue] | None = None

    def dump(self):
        return self.model_dump(exclude_none=True)


import datetime
from pydantic import (
    BaseModel, 
    StrictStr, 
    StrictInt,
    StrictBool,
)
from typing import Literal
from .ec2_tag import EC2Tag
from .ec2_operator import EC2Operator
from .ec2_attachment import EC2Attachment

EC2VolumeType = Literal[
    "standard",
    "io1",
    "io2",
    "gp2",
    "sc1",
    "st1",
    "gp3",
]

EC2SSEType = Literal[
    "sse-ebs",
    "sse-kms",
    "none",
]

EC2State = Literal[
    "creating",
    "available",
    "in-use",
    "deleting",
    "deleted",
    "error"
]


class EC2Volume(BaseModel):
    OutpostArn: StrictStr | None = None
    Iops: StrictInt
    Tags: list[EC2Tag] | None = None
    VolumeType: EC2VolumeType
    FastRestored: StrictBool
    MultiAttachEnabled: StrictBool
    Throughput: StrictInt
    SseType: EC2SSEType
    Operator: EC2Operator | None = None
    VolumeInitializationRate: StrictInt
    VolumeId: StrictStr
    Size: StrictInt
    SnapshotId: StrictStr
    AvailabilityZone: StrictStr
    State: EC2State
    CreateTime: datetime.datetime
    Attachments: list[EC2Attachment] | None = None
    Encrypted: StrictBool = False
    KmsKeyId: StrictStr | None = None




import datetime
from pydantic import BaseModel, StrictStr, StrictFloat, StrictInt
from typing import Literal
from .cloudwatch_message import CloudWatchMessage


CloudWatchMetricDataResultStatusCode = Literal[
    "Complete",
    "InternalError",
    "PartialData",
    "Forbidden"
]


class CloudWatchMetricDataResult(BaseModel):
    Id: StrictStr
    Label: StrictStr
    Timestamps: list[datetime.datetime]
    Values: list[StrictInt | StrictFloat]
    StatusCode: CloudWatchMetricDataResultStatusCode
    Messages: list[CloudWatchMessage] | None =None



from pydantic import (
    BaseModel,
    StrictStr,
    StrictBool
)

from typing import Literal, Optional


class AWSs3PutObjectResponse(BaseModel):
    Expiration: StrictStr
    ETag: StrictStr
    ChecksumCRC32: StrictStr
    ChecksumCRC32C: StrictStr
    ChecksumSHA1: StrictStr
    ChecksumSHA256: StrictStr
    ServerSideEncryption: Literal[
        'AES256',
        'aws:kms',
        'aws:kms:dsse'
    ]
    VersionId: StrictStr
    SSECustomerAlgorithm: StrictStr
    SSDCustomerMD5: StrictStr
    SSEKMSKeyId: Optional[StrictStr]=None
    SSEKMSEncryptionContext: Optional[StrictStr]=None
    BucketKeyEnabled: StrictBool
    RequestCharged: Optional[
        Literal['requester']
    ]=None


from enum import Enum
from typing import Literal
from .cloudwatch_aws_services_map import (
    CloudWatchServiceName,
    CloudWatchAWSServicesMap,
)

CloudWatchAPIOperationName = Literal[
    "accountprovisionedreadcapacityunits",
    "accountprovisionedwritecapacityunits",
    "cachesperregion",
    "channel",
    "createlogstream",
    "deletesession",
    "describeaccesspoints",
    "describeaccountattributes",
    "describeaccountattributes",
    "describeaccountlimits",
    "describeaccountlimits",
    "describeacls",
    "describeactivereceiptruleset",
    "describeaddresses",
    "describeaddressesattribute",
    "describealarms",
    "describeapplications",
    "describeapplicationversions",
    "describeautoscalinggroups",
    "describeautoscalinginstances",
    "describebackupvault",
    "describebluegreendeployments",
    "describebyoipcidrs",
    "describecacheparametergroups",
    "describecachesubnetgroups",
    "describecapacityreservationfleets",
    "describecapacityreservations",
    "describecarriergateways",
    "describeclientvpnendpoints",
    "describeclusters",
    "describeclusters",
    "describeclusters",
    "describeclustersecuritygroups",
    "describeclustersnapshots",
    "describeclustersubnetgroups",
    "describecoippools",
    "describecompliancebyconfigrule",
    "describecompliancebyresource",
    "describecomputeenvironments",
    "describeconfigruleevaluationstatus",
    "describeconfigrules",
    "describeconfigurationrecorders",
    "describeconfigurationrecorderstatus",
    "describecustomkeystores",
    "describedbclusterendpoints",
    "describedbclusterparametergroups",
    "describedbclusters",
    "describedbclustersnapshots",
    "describedbinstanceautomatedbackups",
    "describedbinstances",
    "describedbrecommendations",
    "describedbsecuritygroups",
    "describedbsnapshots",
    "describedbsubnetgroups",
    "describedestinations",
    "describedhcpoptions",
    "describedirectories",
    "describedocument",
    "describedocumentpermission",
    "describeegressonlyinternetgateways",
    "describeendpointauthorization",
    "describeenvironments",
    "describeeventbus",
    "describeevents",
    "describeeventsubscriptions",
    "describeeventsubscriptions",
    "describeeventsubscriptions",
    "describefleetattributes",
    "describefleets",
    "describefleets",
    "describeflowlogs",
    "describefpgaimages",
    "describegamesessionqueues",
    "describeglobalclusters",
    "describehostreservations",
    "describeimagebuilders",
    "describeimages",
    "describeinsightrules",
    "describeinstanceconnectendpoints",
    "describeinstanceinformation",
    "describeinstances",
    "describeinstancestatus",
    "describeinstancetypes",
    "describeinternetgateways",
    "describeipampools",
    "describeipamresourcediscoveries",
    "describeipamresourcediscoveryassociations",
    "describeipams",
    "describeipamscopes",
    "describejobdefinitions",
    "describejobqueues",
    "describekey",
    "describelaunchconfigurations",
    "describelaunchtemplateversions",
    "describelimits",
    "describelocalgatewayroutetables",
    "describeloggroups",
    "describelogstreams",
    "describemaintenancewindows",
    "describematchmakingconfigurations",
    "describemetricfilters",
    "describenatgateways",
    "describenetworkinsightsaccessscopeanalyses",
    "describenetworkinsightsaccessscopes",
    "describenetworkinsightsanalyses",
    "describenetworkinsightspaths",
    "describenetworkinterfaces",
    "describeoptiongroups",
    "describeorganizationsaccess",
    "describeparametergroups",
    "describeparameters",
    "describependingaggregationrequests",
    "describependingmaintenanceactions",
    "describeplacementgroups",
    "describeregions",
    "describeregistry",
    "describeremediationconfigurations",
    "describereplicationgroups",
    "describereplicationinstances",
    "describereplicationsubnetgroups",
    "describerepositories",
    "describereservedcachenodes",
    "describereserveddbinstances",
    "describereservedinstances",
    "describereservedinstancesmodifications",
    "describereservednodes",
    "describeresourcepolicies",
    "describescalabletargets",
    "describescalingpolicies",
    "describesecret",
    "describesecuritygrouprules",
    "describesecuritygroups",
    "describeserverlesscaches",
    "describesnapshots",
    "describesnapshots",
    "describesnapshots",
    "describesnapshotschedules",
    "describespotfleetrequests",
    "describespotinstancerequests",
    "describesslpolicies",
    "describestacks",
    "describestacks",
    "describestandardscontrols",
    "describesubnetgroups",
    "describesubnets",
    "describesubscriptionfilters",
    "describetags",
    "describetags",
    "describetargetgroups",
    "describetenantdatabases",
    "describetrafficmirrortargets",
    "describetrails",
    "describetransitgatewayattachments",
    "describetransitgatewayconnectpeers",
    "describetransitgatewayconnects",
    "describetransitgatewayroutetableannouncements",
    "describetransitgatewayroutetables",
    "describetransitgateways",
    "describetransitgatewayvpcattachments",
    "describeusagelimits",
    "describeusers",
    "describeusers",
    "describeverifiedaccessgroups",
    "describeverifiedaccesstrustproviders",
    "describevpcblockpublicaccessexclusions",
    "describevpcblockpublicaccessoptions",
    "describevpcendpointconnectionnotifications",
    "describevpcendpoints",
    "describevpcendpointserviceconfigurations",
    "describevpcendpointservices",
    "describevpcs",
    "describevpnconnections",
    "describevpngateways",
    "describeworkspaces",
    "exportautoscalinggrouprecommendations",
    "exportebsvolumerecommendations",
    "exportec2instancerecommendations",
    "exportecsservicerecommendations",
    "exportlambdafunctionrecommendations",
    "exportlicenserecommendations",
    "exportrdsdatabaserecommendations",
    "getaccount",
    "getaccountsetting",
    "getaccountsettings",
    "getaccountsettings20160819",
    "getaccountstatus",
    "getadministratoraccount",
    "getallowedimagessettings",
    "getapis",
    "getapps",
    "getbackupvaultaccesspolicy",
    "getblockpublicaccessconfiguration",
    "getblockpublicaccessconfiguration",
    "getblockpublicaccessconfiguration",
    "getblueprints",
    "getbucketacl",
    "getbucketcors",
    "getbucketencryption",
    "getbucketintelligenttieringconfiguration",
    "getbucketlifecycle",
    "getbucketlocation",
    "getbucketlogging",
    "getbucketmetricsconfiguration",
    "getbucketnotification",
    "getbucketobjectlockconfiguration",
    "getbucketownershipcontrols",
    "getbucketreplication",
    "getbucketrequestpayment",
    "getbuckets",
    "getbucketversioning",
    "getbucketwebsite",
    "getcertificates",
    "getcompliancedetailsbyconfigrule",
    "getcontainerservices",
    "getdatabases",
    "getdatacatalog",
    "getdeploymentstrategy",
    "getdetector",
    "getdetectors",
    "getdiscoveredresourcecounts",
    "getdomainnames",
    "getebsencryptionbydefault",
    "getenabledstandards",
    "getencryptionconfig",
    "getenrollmentstatus",
    "getentitytypes",
    "getenvironmentstatus",
    "geteventtypes",
    "getexternalmodels",
    "getfindings",
    "getfunction20150331v2",
    "getfunctionurlconfig",
    "getimageblockpublicaccessstate",
    "getinsightselectors",
    "getinstancemetadatadefaults",
    "getinvitationscount",
    "getkeypolicy",
    "getlifecyclepolicies",
    "getmaciesession",
    "getmalwarescansettings",
    "getmanagedprefixlistentries",
    "getmasteraccount",
    "getmetricdata",
    "getmetricstatistics",
    "getoutcomes",
    "getparameter",
    "getpolicy20150331",
    "getrdsdatabaserecommendations",
    "getrecommendationsummaries",
    "getregistrypolicy",
    "getregistryscanningconfiguration",
    "getrelationaldatabases",
    "getresolverrule",
    "getresourcepolicy",
    "getresources",
    "getresourceshares",
    "getsecretvalue",
    "getservicequota",
    "getservicesetting",
    "getsnapshotblockpublicaccessstate",
    "getstackpolicy",
    "getstaticips",
    "gettemplate",
    "gettemplatesummary",
    "gettopicattributes",
    "gettrailstatus",
    "getvariables",
    "getvpclinks",
    "getworkgroup",
    "keysperaccount",
    "listaccesspoints",
    "listagents",
    "listalarmmodels",
    "listaliases",
    "listaliases",
    "listanalyzers",
    "listapidestinations",
    "listapplications",
    "listapplications",
    "listapplications",
    "listapplications",
    "listapplications",
    "listapplications",
    "listarchiverules",
    "listarchives",
    "listassetmodels",
    "listassociations",
    "listattributegroups",
    "listbackupplans",
    "listbots",
    "listbridges",
    "listbuckets",
    "listcacertificates",
    "listcallanalyticsjobs",
    "listcertificateauthorities",
    "listcertificates",
    "listcertificates",
    "listcertificates",
    "listchangesets",
    "listchannels",
    "listclusters",
    "listclusters",
    "listclusters",
    "listclusters",
    "listcoderepositories",
    "listcodesigningconfigs",
    "listcollaborations",
    "listcomponents",
    "listcomponents",
    "listconfigs",
    "listconfigurationrecorders",
    "listconfigurationsets",
    "listconnectordefinitions",
    "listcontactlists",
    "listcontainerrecipes",
    "listcontainers",
    "listcustommetrics",
    "listdatacatalogs",
    "listdatasets",
    "listdatasets",
    "listdatasets",
    "listdatastores",
    "listdeploymentstrategies",
    "listdetectormodels",
    "listdetectors",
    "listdevicedefinitions",
    "listdimensions",
    "listdiscoveredresources",
    "listdistributionconfigurations",
    "listdocumentclassifiers",
    "listdocuments",
    "listdomains",
    "listdomains",
    "listdomains",
    "listdominantlanguagedetectionjobs",
    "listemailidentities",
    "listemailtemplates",
    "listenabledproductsforimport",
    "listendpointconfigs",
    "listendpoints",
    "listendpoints",
    "listentitiesdetectionjobs",
    "listentityrecognizers",
    "listeventbuses",
    "listeventintegrations",
    "listeventsdetectionjobs",
    "listextensionassociations",
    "listfeaturegroups",
    "listfilters",
    "listfirewalldomainlists",
    "listfirewallpolicies",
    "listfirewalls",
    "listfleetmetrics",
    "listflows",
    "listflows",
    "listflywheels",
    "listfunctiondefinitions",
    "listfunctions20150331",
    "listfunctionurlconfigs",
    "listfuotatasks",
    "listgameservergroups",
    "listgateways",
    "listgateways",
    "listgrants",
    "listgraphs",
    "listgraphs",
    "listgraphsnapshots",
    "listgroups",
    "listgroups",
    "listimagerecipes",
    "listimages",
    "listimages",
    "listinferenceexperiments",
    "listinfrastructureconfigurations",
    "listinputs",
    "listinstances",
    "listinvitations",
    "listkeyphrasesdetectionjobs",
    "listkeyrotations",
    "listkeys",
    "listlanguagemodels",
    "listlayers20181031",
    "listledgers",
    "listlocations",
    "listloggerdefinitions",
    "listloggingconfigurations",
    "listmaps",
    "listmedicalscribejobs",
    "listmedicaltranscriptionjobs",
    "listmedicalvocabularies",
    "listmembers",
    "listmembers",
    "listmetrics",
    "listmissionprofiles",
    "listmitigationactions",
    "listmulticastgroups",
    "listmultiregionaccesspoints",
    "listnotebookinstancelifecycleconfigs",
    "listnotificationhubs",
    "listnotificationrules",
    "listorganizations",
    "listparalleldata",
    "listphonenumbersv2",
    "listpiientitiesdetectionjobs",
    "listpipelines",
    "listpipelines",
    "listplaybackconfigurations",
    "listplaybackkeypairs",
    "listpolicies",
    "listportals",
    "listpreparedstatements",
    "listpresets",
    "listprofiles",
    "listprojects",
    "listprojects",
    "listprojects",
    "listpublishingdestinations",
    "listreceiptfilters",
    "listreceiptrulesets",
    "listrecipes",
    "listrecordingconfigurations",
    "listrecoverypointsbybackupvault",
    "listreportgroups",
    "listrepositories",
    "listrepositories",
    "listresolverendpoints",
    "listresolverquerylogconfigs",
    "listresourcedatasync",
    "listresourcedefinitions",
    "listresourcetags",
    "listrobotapplications",
    "listrolealiases",
    "listrules",
    "listrulesets",
    "listscheduledaudits",
    "listschedulegroups",
    "listschedules",
    "listschedules",
    "listschedulingpolicies",
    "listsecrets",
    "listsecurityconfigurations",
    "listsecurityconfigurations",
    "listsecurityconfigurations",
    "listsecurityprofiles",
    "listsentimentdetectionjobs",
    "listservers",
    "listserviceprofiles",
    "listservices",
    "listsignalingchannels",
    "listsimulationapplications",
    "listsites",
    "liststackresources",
    "liststacks",
    "liststacksets",
    "liststoragelensgroups",
    "liststreams",
    "listsubscriptions",
    "listsubscriptionsbytopic",
    "listtags",
    "listtagsforresource",
    "listtagsforresource",
    "listtagsforresource",
    "listtagsforresource",
    "listtagsforresource",
    "listtagsforresource",
    "listtagsforresource",
    "listtargetsbyrule",
    "listtasks",
    "listtemplates",
    "listterminologies",
    "listtexttranslationjobs",
    "listthings",
    "listthreatintelsets",
    "listtopicruledestinations",
    "listtopicsdetectionjobs",
    "listtrackers",
    "listtypes",
    "listuserpools",
    "listvaults",
    "listversionsbyfunction20150331",
    "listvirtualclusters",
    "listvocabularies",
    "listworkgroups",
    "listworkspaces",
    "listworkspaces",
    "listworkteams",
    "memcachednodespercluster",
    "nodesperregion",
    "numberofmessagespublishedperaccount",
    "parametergroup",
    "putevaluations",
    "putlogevents",
    "selectresourceconfig",
    "snapshotsperdaypercache",
    "stackinstanceoperations",
    "subnetspersubnetgroup",
    "user",
    "usergroup",
    "userpoolread",
    "usersperusergroup",

]


class CloudWatchAPIOperation(Enum):
    ACCOUNT_PROVISIONED_READ_CAPACITY_UNITS="AccountProvisionedReadCapacityUnits"
    ACCOUNT_PROVISIONED_WRITE_CAPACITY_UNITS="AccountProvisionedWriteCapacityUnits"
    CACHES_PER_REGION="CachesPerRegion"
    CHANNEL="Channel"
    CREATE_LOG_STREAM="CreateLogStream"
    DELETE_SESSION="DeleteSession"
    DESCRIBE_ACCESS_POINTS="DescribeAccessPoints"
    DESCRIBE_ACCOUNT_ATTRIBUTES="DescribeAccountAttributes"
    DESCRIBE_ACCOUNT_ATTRIBUTES="DescribeAccountAttributes"
    DESCRIBE_ACCOUNT_LIMITS="DescribeAccountLimits"
    DESCRIBE_ACCOUNT_LIMITS="DescribeAccountLimits"
    DESCRIBE_ACTIVE_RECEIPT_RULE_SET="DescribeActiveReceiptRuleSet"
    DESCRIBE_AC_LS="DescribeACLs"
    DESCRIBE_ADDRESSES="DescribeAddresses"
    DESCRIBE_ADDRESSES_ATTRIBUTE="DescribeAddressesAttribute"
    DESCRIBE_ALARMS="DescribeAlarms"
    DESCRIBE_APPLICATIONS="DescribeApplications"
    DESCRIBE_APPLICATION_VERSIONS="DescribeApplicationVersions"
    DESCRIBE_AUTO_SCALING_GROUPS="DescribeAutoScalingGroups"
    DESCRIBE_AUTO_SCALING_INSTANCES="DescribeAutoScalingInstances"
    DESCRIBE_BACKUP_VAULT="DescribeBackupVault"
    DESCRIBE_BLUE_GREEN_DEPLOYMENTS="DescribeBlueGreenDeployments"
    DESCRIBE_BYOIP_CIDRS="DescribeByoipCidrs"
    DESCRIBE_CACHE_PARAMETER_GROUPS="DescribeCacheParameterGroups"
    DESCRIBE_CACHE_SUBNET_GROUPS="DescribeCacheSubnetGroups"
    DESCRIBE_CAPACITY_RESERVATIONS="DescribeCapacityReservations"
    DESCRIBE_CAPACITY_RESERVATION_FLEETS="DescribeCapacityReservationFleets"
    DESCRIBE_CARRIER_GATEWAYS="DescribeCarrierGateways"
    DESCRIBE_CLIENT_VPN_ENDPOINTS="DescribeClientVpnEndpoints"
    DESCRIBE_CLUSTERS="DescribeClusters"
    DESCRIBE_CLUSTERS="DescribeClusters"
    DESCRIBE_CLUSTERS="DescribeClusters"
    DESCRIBE_CLUSTER_SECURITY_GROUPS="DescribeClusterSecurityGroups"
    DESCRIBE_CLUSTER_SNAPSHOTS="DescribeClusterSnapshots"
    DESCRIBE_CLUSTER_SUBNET_GROUPS="DescribeClusterSubnetGroups"
    DESCRIBE_COIP_POOLS="DescribeCoipPools"
    DESCRIBE_COMPLIANCE_BY_CONFIG_RULE="DescribeComplianceByConfigRule"
    DESCRIBE_COMPLIANCE_BY_RESOURCE="DescribeComplianceByResource"
    DESCRIBE_COMPUTE_ENVIRONMENTS="DescribeComputeEnvironments"
    DESCRIBE_CONFIGURATION_RECORDERS="DescribeConfigurationRecorders"
    DESCRIBE_CONFIGURATION_RECORDER_STATUS="DescribeConfigurationRecorderStatus"
    DESCRIBE_CONFIG_RULES="DescribeConfigRules"
    DESCRIBE_CONFIG_RULE_EVALUATION_STATUS="DescribeConfigRuleEvaluationStatus"
    DESCRIBE_CUSTOM_KEY_STORES="DescribeCustomKeyStores"
    DESCRIBE_DB_CLUSTERS="DescribeDBClusters"
    DESCRIBE_DB_CLUSTER_ENDPOINTS="DescribeDBClusterEndpoints"
    DESCRIBE_DB_CLUSTER_PARAMETER_GROUPS="DescribeDBClusterParameterGroups"
    DESCRIBE_DB_CLUSTER_SNAPSHOTS="DescribeDBClusterSnapshots"
    DESCRIBE_DB_INSTANCES="DescribeDBInstances"
    DESCRIBE_DB_INSTANCE_AUTOMATED_BACKUPS="DescribeDBInstanceAutomatedBackups"
    DESCRIBE_DB_RECOMMENDATIONS="DescribeDBRecommendations"
    DESCRIBE_DB_SECURITY_GROUPS="DescribeDBSecurityGroups"
    DESCRIBE_DB_SNAPSHOTS="DescribeDBSnapshots"
    DESCRIBE_DB_SUBNET_GROUPS="DescribeDBSubnetGroups"
    DESCRIBE_DESTINATIONS="DescribeDestinations"
    DESCRIBE_DHCP_OPTIONS="DescribeDhcpOptions"
    DESCRIBE_DIRECTORIES="DescribeDirectories"
    DESCRIBE_DOCUMENT="DescribeDocument"
    DESCRIBE_DOCUMENT_PERMISSION="DescribeDocumentPermission"
    DESCRIBE_EGRESS_ONLY_INTERNET_GATEWAYS="DescribeEgressOnlyInternetGateways"
    DESCRIBE_ENDPOINT_AUTHORIZATION="DescribeEndpointAuthorization"
    DESCRIBE_ENVIRONMENTS="DescribeEnvironments"
    DESCRIBE_EVENTS="DescribeEvents"
    DESCRIBE_EVENT_BUS="DescribeEventBus"
    DESCRIBE_EVENT_SUBSCRIPTIONS="DescribeEventSubscriptions"
    DESCRIBE_EVENT_SUBSCRIPTIONS="DescribeEventSubscriptions"
    DESCRIBE_EVENT_SUBSCRIPTIONS="DescribeEventSubscriptions"
    DESCRIBE_FLEETS="DescribeFleets"
    DESCRIBE_FLEETS="DescribeFleets"
    DESCRIBE_FLEET_ATTRIBUTES="DescribeFleetAttributes"
    DESCRIBE_FLOW_LOGS="DescribeFlowLogs"
    DESCRIBE_FPGA_IMAGES="DescribeFpgaImages"
    DESCRIBE_GAME_SESSION_QUEUES="DescribeGameSessionQueues"
    DESCRIBE_GLOBAL_CLUSTERS="DescribeGlobalClusters"
    DESCRIBE_HOST_RESERVATIONS="DescribeHostReservations"
    DESCRIBE_IMAGES="DescribeImages"
    DESCRIBE_IMAGE_BUILDERS="DescribeImageBuilders"
    DESCRIBE_INSIGHT_RULES="DescribeInsightRules"
    DESCRIBE_INSTANCES="DescribeInstances"
    DESCRIBE_INSTANCE_CONNECT_ENDPOINTS="DescribeInstanceConnectEndpoints"
    DESCRIBE_INSTANCE_INFORMATION="DescribeInstanceInformation"
    DESCRIBE_INSTANCE_STATUS="DescribeInstanceStatus"
    DESCRIBE_INSTANCE_TYPES="DescribeInstanceTypes"
    DESCRIBE_INTERNET_GATEWAYS="DescribeInternetGateways"
    DESCRIBE_IPAMS="DescribeIpams"
    DESCRIBE_IPAM_POOLS="DescribeIpamPools"
    DESCRIBE_IPAM_RESOURCE_DISCOVERIES="DescribeIpamResourceDiscoveries"
    DESCRIBE_IPAM_RESOURCE_DISCOVERY_ASSOCIATIONS="DescribeIpamResourceDiscoveryAssociations"
    DESCRIBE_IPAM_SCOPES="DescribeIpamScopes"
    DESCRIBE_JOB_DEFINITIONS="DescribeJobDefinitions"
    DESCRIBE_JOB_QUEUES="DescribeJobQueues"
    DESCRIBE_KEY="DescribeKey"
    DESCRIBE_LAUNCH_CONFIGURATIONS="DescribeLaunchConfigurations"
    DESCRIBE_LAUNCH_TEMPLATE_VERSIONS="DescribeLaunchTemplateVersions"
    DESCRIBE_LIMITS="DescribeLimits"
    DESCRIBE_LOCAL_GATEWAY_ROUTE_TABLES="DescribeLocalGatewayRouteTables"
    DESCRIBE_LOG_GROUPS="DescribeLogGroups"
    DESCRIBE_LOG_STREAMS="DescribeLogStreams"
    DESCRIBE_MAINTENANCE_WINDOWS="DescribeMaintenanceWindows"
    DESCRIBE_MATCHMAKING_CONFIGURATIONS="DescribeMatchmakingConfigurations"
    DESCRIBE_METRIC_FILTERS="DescribeMetricFilters"
    DESCRIBE_NAT_GATEWAYS="DescribeNatGateways"
    DESCRIBE_NETWORK_INSIGHTS_ACCESS_SCOPES="DescribeNetworkInsightsAccessScopes"
    DESCRIBE_NETWORK_INSIGHTS_ACCESS_SCOPE_ANALYSES="DescribeNetworkInsightsAccessScopeAnalyses"
    DESCRIBE_NETWORK_INSIGHTS_ANALYSES="DescribeNetworkInsightsAnalyses"
    DESCRIBE_NETWORK_INSIGHTS_PATHS="DescribeNetworkInsightsPaths"
    DESCRIBE_NETWORK_INTERFACES="DescribeNetworkInterfaces"
    DESCRIBE_OPTION_GROUPS="DescribeOptionGroups"
    DESCRIBE_ORGANIZATIONS_ACCESS="DescribeOrganizationsAccess"
    DESCRIBE_PARAMETERS="DescribeParameters"
    DESCRIBE_PARAMETER_GROUPS="DescribeParameterGroups"
    DESCRIBE_PENDING_AGGREGATION_REQUESTS="DescribePendingAggregationRequests"
    DESCRIBE_PENDING_MAINTENANCE_ACTIONS="DescribePendingMaintenanceActions"
    DESCRIBE_PLACEMENT_GROUPS="DescribePlacementGroups"
    DESCRIBE_REGIONS="DescribeRegions"
    DESCRIBE_REGISTRY="DescribeRegistry"
    DESCRIBE_REMEDIATION_CONFIGURATIONS="DescribeRemediationConfigurations"
    DESCRIBE_REPLICATION_GROUPS="DescribeReplicationGroups"
    DESCRIBE_REPLICATION_INSTANCES="DescribeReplicationInstances"
    DESCRIBE_REPLICATION_SUBNET_GROUPS="DescribeReplicationSubnetGroups"
    DESCRIBE_REPOSITORIES="DescribeRepositories"
    DESCRIBE_RESERVED_CACHE_NODES="DescribeReservedCacheNodes"
    DESCRIBE_RESERVED_DB_INSTANCES="DescribeReservedDBInstances"
    DESCRIBE_RESERVED_INSTANCES="DescribeReservedInstances"
    DESCRIBE_RESERVED_INSTANCES_MODIFICATIONS="DescribeReservedInstancesModifications"
    DESCRIBE_RESERVED_NODES="DescribeReservedNodes"
    DESCRIBE_RESOURCE_POLICIES="DescribeResourcePolicies"
    DESCRIBE_SCALABLE_TARGETS="DescribeScalableTargets"
    DESCRIBE_SCALING_POLICIES="DescribeScalingPolicies"
    DESCRIBE_SECRET="DescribeSecret"
    DESCRIBE_SECURITY_GROUPS="DescribeSecurityGroups"
    DESCRIBE_SECURITY_GROUP_RULES="DescribeSecurityGroupRules"
    DESCRIBE_SERVERLESS_CACHES="DescribeServerlessCaches"
    DESCRIBE_SNAPSHOTS="DescribeSnapshots"
    DESCRIBE_SNAPSHOTS="DescribeSnapshots"
    DESCRIBE_SNAPSHOTS="DescribeSnapshots"
    DESCRIBE_SNAPSHOT_SCHEDULES="DescribeSnapshotSchedules"
    DESCRIBE_SPOT_FLEET_REQUESTS="DescribeSpotFleetRequests"
    DESCRIBE_SPOT_INSTANCE_REQUESTS="DescribeSpotInstanceRequests"
    DESCRIBE_SSL_POLICIES="DescribeSSLPolicies"
    DESCRIBE_STACKS="DescribeStacks"
    DESCRIBE_STACKS="DescribeStacks"
    DESCRIBE_STANDARDS_CONTROLS="DescribeStandardsControls"
    DESCRIBE_SUBNETS="DescribeSubnets"
    DESCRIBE_SUBNET_GROUPS="DescribeSubnetGroups"
    DESCRIBE_SUBSCRIPTION_FILTERS="DescribeSubscriptionFilters"
    DESCRIBE_TAGS="DescribeTags"
    DESCRIBE_TAGS="DescribeTags"
    DESCRIBE_TARGET_GROUPS="DescribeTargetGroups"
    DESCRIBE_TENANT_DATABASES="DescribeTenantDatabases"
    DESCRIBE_TRAFFIC_MIRROR_TARGETS="DescribeTrafficMirrorTargets"
    DESCRIBE_TRAILS="DescribeTrails"
    DESCRIBE_TRANSIT_GATEWAYS="DescribeTransitGateways"
    DESCRIBE_TRANSIT_GATEWAY_ATTACHMENTS="DescribeTransitGatewayAttachments"
    DESCRIBE_TRANSIT_GATEWAY_CONNECTS="DescribeTransitGatewayConnects"
    DESCRIBE_TRANSIT_GATEWAY_CONNECT_PEERS="DescribeTransitGatewayConnectPeers"
    DESCRIBE_TRANSIT_GATEWAY_ROUTE_TABLES="DescribeTransitGatewayRouteTables"
    DESCRIBE_TRANSIT_GATEWAY_ROUTE_TABLE_ANNOUNCEMENTS="DescribeTransitGatewayRouteTableAnnouncements"
    DESCRIBE_TRANSIT_GATEWAY_VPC_ATTACHMENTS="DescribeTransitGatewayVpcAttachments"
    DESCRIBE_USAGE_LIMITS="DescribeUsageLimits"
    DESCRIBE_USERS="DescribeUsers"
    DESCRIBE_USERS="DescribeUsers"
    DESCRIBE_VERIFIED_ACCESS_GROUPS="DescribeVerifiedAccessGroups"
    DESCRIBE_VERIFIED_ACCESS_TRUST_PROVIDERS="DescribeVerifiedAccessTrustProviders"
    DESCRIBE_VPCS="DescribeVpcs"
    DESCRIBE_VPC_BLOCK_PUBLIC_ACCESS_EXCLUSIONS="DescribeVpcBlockPublicAccessExclusions"
    DESCRIBE_VPC_BLOCK_PUBLIC_ACCESS_OPTIONS="DescribeVpcBlockPublicAccessOptions"
    DESCRIBE_VPC_ENDPOINTS="DescribeVpcEndpoints"
    DESCRIBE_VPC_ENDPOINT_CONNECTION_NOTIFICATIONS="DescribeVpcEndpointConnectionNotifications"
    DESCRIBE_VPC_ENDPOINT_SERVICES="DescribeVpcEndpointServices"
    DESCRIBE_VPC_ENDPOINT_SERVICE_CONFIGURATIONS="DescribeVpcEndpointServiceConfigurations"
    DESCRIBE_VPN_CONNECTIONS="DescribeVpnConnections"
    DESCRIBE_VPN_GATEWAYS="DescribeVpnGateways"
    DESCRIBE_WORKSPACES="DescribeWorkspaces"
    EXPORT_AUTO_SCALING_GROUP_RECOMMENDATIONS="ExportAutoScalingGroupRecommendations"
    EXPORT_EBS_VOLUME_RECOMMENDATIONS="ExportEBSVolumeRecommendations"
    EXPORT_EC2_INSTANCE_RECOMMENDATIONS="ExportEC2InstanceRecommendations"
    EXPORT_ECS_SERVICE_RECOMMENDATIONS="ExportECSServiceRecommendations"
    EXPORT_LAMBDA_FUNCTION_RECOMMENDATIONS="ExportLambdaFunctionRecommendations"
    EXPORT_LICENSE_RECOMMENDATIONS="ExportLicenseRecommendations"
    EXPORT_RDS_DATABASE_RECOMMENDATIONS="ExportRDSDatabaseRecommendations"
    GET_ACCOUNT="GetAccount"
    GET_ACCOUNT_SETTING="GetAccountSetting"
    GET_ACCOUNT_SETTINGS20160819="GetAccountSettings20160819"
    GET_ACCOUNT_SETTINGS="GetAccountSettings"
    GET_ACCOUNT_STATUS="GetAccountStatus"
    GET_ADMINISTRATOR_ACCOUNT="GetAdministratorAccount"
    GET_ALLOWED_IMAGES_SETTINGS="GetAllowedImagesSettings"
    GET_APIS="GetApis"
    GET_APPS="GetApps"
    GET_BACKUP_VAULT_ACCESS_POLICY="GetBackupVaultAccessPolicy"
    GET_BLOCK_PUBLIC_ACCESS_CONFIGURATION="GetBlockPublicAccessConfiguration"
    GET_BLOCK_PUBLIC_ACCESS_CONFIGURATION="GetBlockPublicAccessConfiguration"
    GET_BLOCK_PUBLIC_ACCESS_CONFIGURATION="GetBlockPublicAccessConfiguration"
    GET_BLUEPRINTS="GetBlueprints"
    GET_BUCKETS="GetBuckets"
    GET_BUCKET_ACL="GetBucketAcl"
    GET_BUCKET_CORS="GetBucketCors"
    GET_BUCKET_ENCRYPTION="GetBucketEncryption"
    GET_BUCKET_INTELLIGENT_TIERING_CONFIGURATION="GetBucketIntelligentTieringConfiguration"
    GET_BUCKET_LIFECYCLE="GetBucketLifecycle"
    GET_BUCKET_LOCATION="GetBucketLocation"
    GET_BUCKET_LOGGING="GetBucketLogging"
    GET_BUCKET_METRICS_CONFIGURATION="GetBucketMetricsConfiguration"
    GET_BUCKET_NOTIFICATION="GetBucketNotification"
    GET_BUCKET_OBJECT_LOCK_CONFIGURATION="GetBucketObjectLockConfiguration"
    GET_BUCKET_OWNERSHIP_CONTROLS="GetBucketOwnershipControls"
    GET_BUCKET_REPLICATION="GetBucketReplication"
    GET_BUCKET_REQUEST_PAYMENT="GetBucketRequestPayment"
    GET_BUCKET_VERSIONING="GetBucketVersioning"
    GET_BUCKET_WEBSITE="GetBucketWebsite"
    GET_CERTIFICATES="GetCertificates"
    GET_COMPLIANCE_DETAILS_BY_CONFIG_RULE="GetComplianceDetailsByConfigRule"
    GET_CONTAINER_SERVICES="GetContainerServices"
    GET_DATABASES="GetDatabases"
    GET_DATA_CATALOG="GetDataCatalog"
    GET_DEPLOYMENT_STRATEGY="GetDeploymentStrategy"
    GET_DETECTOR="GetDetector"
    GET_DETECTORS="GetDetectors"
    GET_DISCOVERED_RESOURCE_COUNTS="GetDiscoveredResourceCounts"
    GET_DOMAIN_NAMES="GetDomainNames"
    GET_EBS_ENCRYPTION_BY_DEFAULT="GetEbsEncryptionByDefault"
    GET_ENABLED_STANDARDS="GetEnabledStandards"
    GET_ENCRYPTION_CONFIG="GetEncryptionConfig"
    GET_ENROLLMENT_STATUS="GetEnrollmentStatus"
    GET_ENTITY_TYPES="GetEntityTypes"
    GET_ENVIRONMENT_STATUS="GetEnvironmentStatus"
    GET_EVENT_TYPES="GetEventTypes"
    GET_EXTERNAL_MODELS="GetExternalModels"
    GET_FINDINGS="GetFindings"
    GET_FUNCTION20150331V2="GetFunction20150331v2"
    GET_FUNCTION_URL_CONFIG="GetFunctionUrlConfig"
    GET_IMAGE_BLOCK_PUBLIC_ACCESS_STATE="GetImageBlockPublicAccessState"
    GET_INSIGHT_SELECTORS="GetInsightSelectors"
    GET_INSTANCE_METADATA_DEFAULTS="GetInstanceMetadataDefaults"
    GET_INVITATIONS_COUNT="GetInvitationsCount"
    GET_KEY_POLICY="GetKeyPolicy"
    GET_LIFECYCLE_POLICIES="GetLifecyclePolicies"
    GET_MACIE_SESSION="GetMacieSession"
    GET_MALWARE_SCAN_SETTINGS="GetMalwareScanSettings"
    GET_MANAGED_PREFIX_LIST_ENTRIES="GetManagedPrefixListEntries"
    GET_MASTER_ACCOUNT="GetMasterAccount"
    GET_METRIC_DATA="GetMetricData"
    GET_METRIC_STATISTICS="GetMetricStatistics"
    GET_OUTCOMES="GetOutcomes"
    GET_PARAMETER="GetParameter"
    GET_POLICY20150331="GetPolicy20150331"
    GET_RDS_DATABASE_RECOMMENDATIONS="GetRDSDatabaseRecommendations"
    GET_RECOMMENDATION_SUMMARIES="GetRecommendationSummaries"
    GET_REGISTRY_POLICY="GetRegistryPolicy"
    GET_REGISTRY_SCANNING_CONFIGURATION="GetRegistryScanningConfiguration"
    GET_RELATIONAL_DATABASES="GetRelationalDatabases"
    GET_RESOLVER_RULE="GetResolverRule"
    GET_RESOURCES="GetResources"
    GET_RESOURCE_POLICY="GetResourcePolicy"
    GET_RESOURCE_SHARES="GetResourceShares"
    GET_SECRET_VALUE="GetSecretValue"
    GET_SERVICE_QUOTA="GetServiceQuota"
    GET_SERVICE_SETTING="GetServiceSetting"
    GET_SNAPSHOT_BLOCK_PUBLIC_ACCESS_STATE="GetSnapshotBlockPublicAccessState"
    GET_STACK_POLICY="GetStackPolicy"
    GET_STATIC_IPS="GetStaticIps"
    GET_TEMPLATE="GetTemplate"
    GET_TEMPLATE_SUMMARY="GetTemplateSummary"
    GET_TOPIC_ATTRIBUTES="GetTopicAttributes"
    GET_TRAIL_STATUS="GetTrailStatus"
    GET_VARIABLES="GetVariables"
    GET_VPC_LINKS="GetVpcLinks"
    GET_WORK_GROUP="GetWorkGroup"
    KEYS_PER_ACCOUNT="KeysPerAccount"
    LIST_ACCESS_POINTS="ListAccessPoints"
    LIST_AGENTS="ListAgents"
    LIST_ALARM_MODELS="ListAlarmModels"
    LIST_ALIASES="ListAliases"
    LIST_ALIASES="ListAliases"
    LIST_ANALYZERS="ListAnalyzers"
    LIST_API_DESTINATIONS="ListApiDestinations"
    LIST_APPLICATIONS="ListApplications"
    LIST_APPLICATIONS="ListApplications"
    LIST_APPLICATIONS="ListApplications"
    LIST_APPLICATIONS="ListApplications"
    LIST_APPLICATIONS="ListApplications"
    LIST_APPLICATIONS="ListApplications"
    LIST_ARCHIVES="ListArchives"
    LIST_ARCHIVE_RULES="ListArchiveRules"
    LIST_ASSET_MODELS="ListAssetModels"
    LIST_ASSOCIATIONS="ListAssociations"
    LIST_ATTRIBUTE_GROUPS="ListAttributeGroups"
    LIST_BACKUP_PLANS="ListBackupPlans"
    LIST_BOTS="ListBots"
    LIST_BRIDGES="ListBridges"
    LIST_BUCKETS="ListBuckets"
    LIST_CALL_ANALYTICS_JOBS="ListCallAnalyticsJobs"
    LIST_CA_CERTIFICATES="ListCACertificates"
    LIST_CERTIFICATES="ListCertificates"
    LIST_CERTIFICATES="ListCertificates"
    LIST_CERTIFICATES="ListCertificates"
    LIST_CERTIFICATE_AUTHORITIES="ListCertificateAuthorities"
    LIST_CHANGE_SETS="ListChangeSets"
    LIST_CHANNELS="ListChannels"
    LIST_CLUSTERS="ListClusters"
    LIST_CLUSTERS="ListClusters"
    LIST_CLUSTERS="ListClusters"
    LIST_CLUSTERS="ListClusters"
    LIST_CODE_REPOSITORIES="ListCodeRepositories"
    LIST_CODE_SIGNING_CONFIGS="ListCodeSigningConfigs"
    LIST_COLLABORATIONS="ListCollaborations"
    LIST_COMPONENTS="ListComponents"
    LIST_COMPONENTS="ListComponents"
    LIST_CONFIGS="ListConfigs"
    LIST_CONFIGURATION_RECORDERS="ListConfigurationRecorders"
    LIST_CONFIGURATION_SETS="ListConfigurationSets"
    LIST_CONNECTOR_DEFINITIONS="ListConnectorDefinitions"
    LIST_CONTACT_LISTS="ListContactLists"
    LIST_CONTAINERS="ListContainers"
    LIST_CONTAINER_RECIPES="ListContainerRecipes"
    LIST_CUSTOM_METRICS="ListCustomMetrics"
    LIST_DATASETS="ListDatasets"
    LIST_DATASETS="ListDatasets"
    LIST_DATASTORES="ListDatastores"
    LIST_DATA_CATALOGS="ListDataCatalogs"
    LIST_DATA_SETS="ListDataSets"
    LIST_DEPLOYMENT_STRATEGIES="ListDeploymentStrategies"
    LIST_DETECTORS="ListDetectors"
    LIST_DETECTOR_MODELS="ListDetectorModels"
    LIST_DEVICE_DEFINITIONS="ListDeviceDefinitions"
    LIST_DIMENSIONS="ListDimensions"
    LIST_DISCOVERED_RESOURCES="ListDiscoveredResources"
    LIST_DISTRIBUTION_CONFIGURATIONS="ListDistributionConfigurations"
    LIST_DOCUMENTS="ListDocuments"
    LIST_DOCUMENT_CLASSIFIERS="ListDocumentClassifiers"
    LIST_DOMAINS="ListDomains"
    LIST_DOMAINS="ListDomains"
    LIST_DOMAINS="ListDomains"
    LIST_DOMINANT_LANGUAGE_DETECTION_JOBS="ListDominantLanguageDetectionJobs"
    LIST_EMAIL_IDENTITIES="ListEmailIdentities"
    LIST_EMAIL_TEMPLATES="ListEmailTemplates"
    LIST_ENABLED_PRODUCTS_FOR_IMPORT="ListEnabledProductsForImport"
    LIST_ENDPOINTS="ListEndpoints"
    LIST_ENDPOINTS="ListEndpoints"
    LIST_ENDPOINT_CONFIGS="ListEndpointConfigs"
    LIST_ENTITIES_DETECTION_JOBS="ListEntitiesDetectionJobs"
    LIST_ENTITY_RECOGNIZERS="ListEntityRecognizers"
    LIST_EVENTS_DETECTION_JOBS="ListEventsDetectionJobs"
    LIST_EVENT_BUSES="ListEventBuses"
    LIST_EVENT_INTEGRATIONS="ListEventIntegrations"
    LIST_EXTENSION_ASSOCIATIONS="ListExtensionAssociations"
    LIST_FEATURE_GROUPS="ListFeatureGroups"
    LIST_FILTERS="ListFilters"
    LIST_FIREWALLS="ListFirewalls"
    LIST_FIREWALL_DOMAIN_LISTS="ListFirewallDomainLists"
    LIST_FIREWALL_POLICIES="ListFirewallPolicies"
    LIST_FLEET_METRICS="ListFleetMetrics"
    LIST_FLOWS="ListFlows"
    LIST_FLOWS="ListFlows"
    LIST_FLYWHEELS="ListFlywheels"
    LIST_FUNCTIONS20150331="ListFunctions20150331"
    LIST_FUNCTION_DEFINITIONS="ListFunctionDefinitions"
    LIST_FUNCTION_URL_CONFIGS="ListFunctionUrlConfigs"
    LIST_FUOTA_TASKS="ListFuotaTasks"
    LIST_GAME_SERVER_GROUPS="ListGameServerGroups"
    LIST_GATEWAYS="ListGateways"
    LIST_GATEWAYS="ListGateways"
    LIST_GRANTS="ListGrants"
    LIST_GRAPHS="ListGraphs"
    LIST_GRAPHS="ListGraphs"
    LIST_GRAPH_SNAPSHOTS="ListGraphSnapshots"
    LIST_GROUPS="ListGroups"
    LIST_GROUPS="ListGroups"
    LIST_IMAGES="ListImages"
    LIST_IMAGES="ListImages"
    LIST_IMAGE_RECIPES="ListImageRecipes"
    LIST_INFERENCE_EXPERIMENTS="ListInferenceExperiments"
    LIST_INFRASTRUCTURE_CONFIGURATIONS="ListInfrastructureConfigurations"
    LIST_INPUTS="ListInputs"
    LIST_INSTANCES="ListInstances"
    LIST_INVITATIONS="ListInvitations"
    LIST_KEYS="ListKeys"
    LIST_KEY_PHRASES_DETECTION_JOBS="ListKeyPhrasesDetectionJobs"
    LIST_KEY_ROTATIONS="ListKeyRotations"
    LIST_LANGUAGE_MODELS="ListLanguageModels"
    LIST_LAYERS20181031="ListLayers20181031"
    LIST_LEDGERS="ListLedgers"
    LIST_LOCATIONS="ListLocations"
    LIST_LOGGER_DEFINITIONS="ListLoggerDefinitions"
    LIST_LOGGING_CONFIGURATIONS="ListLoggingConfigurations"
    LIST_MAPS="ListMaps"
    LIST_MEDICAL_SCRIBE_JOBS="ListMedicalScribeJobs"
    LIST_MEDICAL_TRANSCRIPTION_JOBS="ListMedicalTranscriptionJobs"
    LIST_MEDICAL_VOCABULARIES="ListMedicalVocabularies"
    LIST_MEMBERS="ListMembers"
    LIST_MEMBERS="ListMembers"
    LIST_METRICS="ListMetrics"
    LIST_MISSION_PROFILES="ListMissionProfiles"
    LIST_MITIGATION_ACTIONS="ListMitigationActions"
    LIST_MULTICAST_GROUPS="ListMulticastGroups"
    LIST_MULTI_REGION_ACCESS_POINTS="ListMultiRegionAccessPoints"
    LIST_NOTEBOOK_INSTANCE_LIFECYCLE_CONFIGS="ListNotebookInstanceLifecycleConfigs"
    LIST_NOTIFICATION_HUBS="ListNotificationHubs"
    LIST_NOTIFICATION_RULES="ListNotificationRules"
    LIST_ORGANIZATIONS="ListOrganizations"
    LIST_PARALLEL_DATA="ListParallelData"
    LIST_PHONE_NUMBERS_V2="ListPhoneNumbersV2"
    LIST_PII_ENTITIES_DETECTION_JOBS="ListPiiEntitiesDetectionJobs"
    LIST_PIPELINES="ListPipelines"
    LIST_PIPELINES="ListPipelines"
    LIST_PLAYBACK_CONFIGURATIONS="ListPlaybackConfigurations"
    LIST_PLAYBACK_KEY_PAIRS="ListPlaybackKeyPairs"
    LIST_POLICIES="ListPolicies"
    LIST_PORTALS="ListPortals"
    LIST_PREPARED_STATEMENTS="ListPreparedStatements"
    LIST_PRESETS="ListPresets"
    LIST_PROFILES="ListProfiles"
    LIST_PROJECTS="ListProjects"
    LIST_PROJECTS="ListProjects"
    LIST_PROJECTS="ListProjects"
    LIST_PUBLISHING_DESTINATIONS="ListPublishingDestinations"
    LIST_RECEIPT_FILTERS="ListReceiptFilters"
    LIST_RECEIPT_RULE_SETS="ListReceiptRuleSets"
    LIST_RECIPES="ListRecipes"
    LIST_RECORDING_CONFIGURATIONS="ListRecordingConfigurations"
    LIST_RECOVERY_POINTS_BY_BACKUP_VAULT="ListRecoveryPointsByBackupVault"
    LIST_REPORT_GROUPS="ListReportGroups"
    LIST_REPOSITORIES="ListRepositories"
    LIST_REPOSITORIES="ListRepositories"
    LIST_RESOLVER_ENDPOINTS="ListResolverEndpoints"
    LIST_RESOLVER_QUERY_LOG_CONFIGS="ListResolverQueryLogConfigs"
    LIST_RESOURCE_DATA_SYNC="ListResourceDataSync"
    LIST_RESOURCE_DEFINITIONS="ListResourceDefinitions"
    LIST_RESOURCE_TAGS="ListResourceTags"
    LIST_ROBOT_APPLICATIONS="ListRobotApplications"
    LIST_ROLE_ALIASES="ListRoleAliases"
    LIST_RULES="ListRules"
    LIST_RULESETS="ListRulesets"
    LIST_SCHEDULED_AUDITS="ListScheduledAudits"
    LIST_SCHEDULES="ListSchedules"
    LIST_SCHEDULES="ListSchedules"
    LIST_SCHEDULE_GROUPS="ListScheduleGroups"
    LIST_SCHEDULING_POLICIES="ListSchedulingPolicies"
    LIST_SECRETS="ListSecrets"
    LIST_SECURITY_CONFIGURATIONS="ListSecurityConfigurations"
    LIST_SECURITY_CONFIGURATIONS="ListSecurityConfigurations"
    LIST_SECURITY_CONFIGURATIONS="ListSecurityConfigurations"
    LIST_SECURITY_PROFILES="ListSecurityProfiles"
    LIST_SENTIMENT_DETECTION_JOBS="ListSentimentDetectionJobs"
    LIST_SERVERS="ListServers"
    LIST_SERVICES="ListServices"
    LIST_SERVICE_PROFILES="ListServiceProfiles"
    LIST_SIGNALING_CHANNELS="ListSignalingChannels"
    LIST_SIMULATION_APPLICATIONS="ListSimulationApplications"
    LIST_SITES="ListSites"
    LIST_STACKS="ListStacks"
    LIST_STACK_RESOURCES="ListStackResources"
    LIST_STACK_SETS="ListStackSets"
    LIST_STORAGE_LENS_GROUPS="ListStorageLensGroups"
    LIST_STREAMS="ListStreams"
    LIST_SUBSCRIPTIONS="ListSubscriptions"
    LIST_SUBSCRIPTIONS_BY_TOPIC="ListSubscriptionsByTopic"
    LIST_TAGS="ListTags"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TAGS_FOR_RESOURCE="ListTagsForResource"
    LIST_TARGETS_BY_RULE="ListTargetsByRule"
    LIST_TASKS="ListTasks"
    LIST_TEMPLATES="ListTemplates"
    LIST_TERMINOLOGIES="ListTerminologies"
    LIST_TEXT_TRANSLATION_JOBS="ListTextTranslationJobs"
    LIST_THINGS="ListThings"
    LIST_THREAT_INTEL_SETS="ListThreatIntelSets"
    LIST_TOPICS_DETECTION_JOBS="ListTopicsDetectionJobs"
    LIST_TOPIC_RULE_DESTINATIONS="ListTopicRuleDestinations"
    LIST_TRACKERS="ListTrackers"
    LIST_TYPES="ListTypes"
    LIST_USER_POOLS="ListUserPools"
    LIST_VAULTS="ListVaults"
    LIST_VERSIONS_BY_FUNCTION20150331="ListVersionsByFunction20150331"
    LIST_VIRTUAL_CLUSTERS="ListVirtualClusters"
    LIST_VOCABULARIES="ListVocabularies"
    LIST_WORKSPACES="ListWorkspaces"
    LIST_WORKSPACES="ListWorkspaces"
    LIST_WORKTEAMS="ListWorkteams"
    LIST_WORK_GROUPS="ListWorkGroups"
    MEMCACHED_NODES_PER_CLUSTER="MemcachedNodesPerCluster"
    NODES_PER_REGION="NodesPerRegion"
    NUMBER_OF_MESSAGES_PUBLISHED_PER_ACCOUNT="NumberOfMessagesPublishedPerAccount"
    PARAMETER_GROUP="ParameterGroup"
    PUT_EVALUATIONS="PutEvaluations"
    PUT_LOG_EVENTS="PutLogEvents"
    SELECT_RESOURCE_CONFIG="SelectResourceConfig"
    SNAPSHOTS_PER_DAY_PER_CACHE="SnapshotsPerDayPerCache"
    STACK_INSTANCE_OPERATIONS="StackInstanceOperations"
    SUBNETS_PER_SUBNET_GROUP="SubnetsPerSubnetGroup"
    USER="User"
    USERS_PER_USER_GROUP="UsersPerUserGroup"
    USER_GROUP="UserGroup"
    USER_POOL_READ="UserPoolRead"


class CloudServiceAPIOpsMap:

    def __init__(self):
        self._api_ops_map: dict[
            CloudWatchAPIOperationName,
            CloudWatchAPIOperation
        ] = {
            service.name.lower().replace(
                '_',
                '-'
            ): service for service in CloudWatchAPIOperation
        }

        self._service_api_ops_map = {
            "DYNAMODB": [
                "accountprovisionedreadcapacityunits",
                "accountprovisionedwritecapacityunits"
            ],
            "ELASTICACHE": [
                "cachesperregion",
                "describecacheparametergroups",
                "describecachesubnetgroups",
                "describereplicationgroups",
                "describereservedcachenodes",
                "describeserverlesscaches",
                "describesnapshots",
                "describeusers",
                "memcachednodespercluster",
                "nodesperregion",
                "parametergroup",
                "snapshotsperdaypercache",
                "subnetspersubnetgroup",
                "user",
                "usergroup",
                "usersperusergroup"
            ],
            "CLOUDTRAIL": [
                "channel",
                "describetrails",
                "getinsightselectors",
                "gettrailstatus",
                "listchannels"
            ],
            "LOGS": [
                "createlogstream",
                "describedestinations",
                "describeloggroups",
                "describelogstreams",
                "describemetricfilters",
                "describeresourcepolicies",
                "describesubscriptionfilters",
                "putlogevents"
            ],
            "CLOUDSHELL": [
                "deletesession",
                "getenvironmentstatus"
            ],
            "EFS": [
                "describeaccesspoints"
            ],
            "EC2": [
                "describeaccountattributes",
                "describeaccountlimits",
                "describeaddresses",
                "describeaddressesattribute",
                "describeautoscalinggroups",
                "describeautoscalinginstances",
                "describebyoipcidrs",
                "describecapacityreservationfleets",
                "describecapacityreservations",
                "describecarriergateways",
                "describeclientvpnendpoints",
                "describecoippools",
                "describedhcpoptions",
                "describeegressonlyinternetgateways",
                "describefleets",
                "describeflowlogs",
                "describefpgaimages",
                "describehostreservations",
                "describeimages",
                "describeinstanceconnectendpoints",
                "describeinstances",
                "describeinstancestatus",
                "describeinstancetypes",
                "describeinternetgateways",
                "describeipampools",
                "describeipamresourcediscoveries",
                "describeipamresourcediscoveryassociations",
                "describeipams",
                "describeipamscopes",
                "describelaunchconfigurations",
                "describelaunchtemplateversions",
                "describelocalgatewayroutetables",
                "describenatgateways",
                "describenetworkinsightsaccessscopeanalyses",
                "describenetworkinsightsaccessscopes",
                "describenetworkinsightsanalyses",
                "describenetworkinsightspaths",
                "describenetworkinterfaces",
                "describeplacementgroups",
                "describeregions",
                "describereservedinstances",
                "describereservedinstancesmodifications",
                "describescalabletargets",
                "describescalingpolicies",
                "describesecuritygrouprules",
                "describesecuritygroups",
                "describesnapshots",
                "describespotfleetrequests",
                "describespotinstancerequests",
                "describesubnets",
                "describetags",
                "describetags",
                "describetrafficmirrortargets",
                "describetransitgatewayattachments",
                "describetransitgatewayconnectpeers",
                "describetransitgatewayconnects",
                "describetransitgatewayroutetableannouncements",
                "describetransitgatewayroutetables",
                "describetransitgateways",
                "describetransitgatewayvpcattachments",
                "describeverifiedaccessgroups",
                "describeverifiedaccesstrustproviders",
                "describevpcblockpublicaccessexclusions",
                "describevpcblockpublicaccessoptions",
                "describevpcendpointconnectionnotifications",
                "describevpcendpoints",
                "describevpcendpointserviceconfigurations",
                "describevpcendpointservices",
                "describevpcs",
                "describevpnconnections",
                "describevpngateways",
                "getallowedimagessettings",
                "getebsencryptionbydefault",
                "getimageblockpublicaccessstate",
                "getinstancemetadatadefaults",
                "getmanagedprefixlistentries",
                "getsnapshotblockpublicaccessstate"
            ],
            "RDS": [
                "describeaccountattributes",
                "describebluegreendeployments",
                "describedbclusterendpoints",
                "describedbclusterparametergroups",
                "describedbclusters",
                "describedbclustersnapshots",
                "describedbinstanceautomatedbackups",
                "describedbinstances",
                "describedbsecuritygroups",
                "describedbsnapshots",
                "describedbsubnetgroups",
                "describeeventsubscriptions",
                "describeglobalclusters",
                "describeoptiongroups",
                "describependingmaintenanceactions",
                "describereserveddbinstances"
            ],
            "CLOUDFORMATION": [
                "describeaccountlimits",
                "describeorganizationsaccess",
                "describestacks",
                "getstackpolicy",
                "gettemplate",
                "gettemplatesummary",
                "listchangesets",
                "liststackresources",
                "liststacks",
                "liststacksets",
                "listtypes",
                "stackinstanceoperations"
            ],
            "MEMORYDB": [
                "describeacls",
                "describeclusters",
                "describeparametergroups",
                "describesnapshots",
                "describesubnetgroups",
                "describeusers"
            ],
            "SES": [
                "describeactivereceiptruleset",
                "getaccount",
                "listconfigurationsets",
                "listcontactlists",
                "listemailidentities",
                "listemailtemplates",
                "listreceiptfilters",
                "listreceiptrulesets"
            ],
            "CLOUDWATCH": [
                "describealarms",
                "describeinsightrules",
                "getmetricdata",
                "getmetricstatistics",
                "listapplications",
                "listmetrics",
                "listtagsforresource"
            ],
            "ELASTIC": [
                "describeapplications",
                "describeapplicationversions",
                "describeenvironments",
                "describesslpolicies",
                "describetargetgroups"
            ],
            "BACKUP": [
                "describebackupvault",
                "getbackupvaultaccesspolicy",
                "listbackupplans",
                "listrecoverypointsbybackupvault",
                "listtags"
            ],
            "CLOUDHSM": [
                "describeclusters"
            ],
            "REDSHIFT": [
                "describeclusters",
                "describeclustersecuritygroups",
                "describeclustersnapshots",
                "describeclustersubnetgroups",
                "describeendpointaccess",
                "describeendpointauthorization",
                "describeeventsubscriptions",
                "describereservednodes",
                "describesnapshotschedules",
                "describeusagelimits"
            ],
            "AWS": [
                "describecompliancebyconfigrule",
                "describecompliancebyresource",
                "describeconfigruleevaluationstatus",
                "describeconfigrules",
                "describeconfigurationrecorders",
                "describeconfigurationrecorderstatus",
                "describependingaggregationrequests",
                "describeremediationconfigurations",
                "getcompliancedetailsbyconfigrule",
                "getdatabases",
                "getdeploymentstrategy",
                "getdiscoveredresourcecounts",
                "getresourcepolicy",
                "listapplications",
                "listconfigurationrecorders",
                "listdatastores",
                "listdeploymentstrategies",
                "listdiscoveredresources",
                "listdomains",
                "listextensionassociations",
                "listnotificationhubs",
                "listrepositories",
                "listtagsforresource",
                "putevaluations",
                "selectresourceconfig"
            ],
            "BATCH": [
                "describecomputeenvironments",
                "describejobdefinitions",
                "describejobqueues",
                "listschedulingpolicies"
            ],
            "KMS": [
                "describecustomkeystores",
                "describekey",
                "getkeypolicy",
                "keysperaccount",
                "listaliases",
                "listgrants",
                "listkeyrotations",
                "listkeys",
                "listresourcetags"
            ],
            "DIRECTORY": [
                "describedirectories"
            ],
            "SSM": [
                "describedocument",
                "describedocumentpermission",
                "describeinstanceinformation",
                "describemaintenancewindows",
                "describeparameters",
                "getparameter",
                "getservicesetting",
                "listassociations",
                "listdocuments",
                "listresourcedatasync"
            ],
            "EVENTBRIDGE": [
                "describeeventbus",
                "listapidestinations",
                "listarchives",
                "listendpoints",
                "listeventbuses",
                "listrules",
                "listtagsforresource",
                "listtargetsbyrule"
            ],
            "DATABASE": [
                "describeeventsubscriptions",
                "describereplicationinstances",
                "describereplicationsubnetgroups"
            ],
            "GAMELIFT": [
                "describefleetattributes",
                "describegamesessionqueues",
                "describematchmakingconfigurations",
                "listaliases",
                "listgameservergroups",
                "listlocations"
            ],
            "APPSTREAM": [
                "describefleets",
                "describeimagebuilders",
                "describestacks"
            ],
            "KINESIS": [
                "describelimits",
                "listapplications",
                "listsignalingchannels",
                "liststreams"
            ],
            "ECR": [
                "describepullthroughcacherules",
                "describeregistry",
                "describerepositories",
                "getaccountsetting",
                "getregistrypolicy",
                "getregistryscanningconfiguration"
            ],
            "SECRETS": [
                "describesecret",
                "getsecretvalue",
                "listsecrets"
            ],
            "SECURITY": [
                "describestandardscontrols",
                "getadministratoraccount",
                "getenabledstandards",
                "getfindings",
                "listenabledproductsforimport",
                "listinvitations",
                "listmembers",
                "listtagsforresource"
            ],
            "WORKSPACES": [
                "describeworkspaces"
            ],
            "COMPUTEOPTIMIZER": [
                "exportautoscalinggrouprecommendations",
                "exportebsvolumerecommendations",
                "exportec2instancerecommendations",
                "exportecsservicerecommendations",
                "exportlambdafunctionrecommendations",
                "exportlicenserecommendations",
                "exportrdsdatabaserecommendations",
                "getenrollmentstatus",
                "getrdsdatabaserecommendations",
                "getrecommendationsummaries"
            ],
            "RESOURCE": [
                "getaccountsettings",
                "getresources",
                "getresourceshares",
                "listgroups"
            ],
            "LAMBDA": [
                "getaccountsettings20160819",
                "getfunction20150331v2",
                "getfunctionurlconfig",
                "getpolicy20150331",
                "listcodesigningconfigs",
                "listfunctions20150331",
                "listfunctionurlconfigs",
                "listlayers20181031",
                "listversionsbyfunction20150331"
            ],
            "AUDIT": [
                "getaccountstatus"
            ],
            "API": [
                "getapis",
                "getdomainnames",
                "getvpclinks"
            ],
            "PINPOINT": [
                "getapps",
                "listtemplates"
            ],
            "EMR": [
                "getblockpublicaccessconfiguration",
                "getblockpublicaccessconfiguration",
                "listapplications",
                "listclusters",
                "listclusters",
                "listsecurityconfigurations",
                "listsecurityconfigurations",
                "listvirtualclusters"
            ],
            "AMAZON": [
                "getblockpublicaccessconfiguration",
                "listbots",
                "listclusters",
                "listdatasets",
                "listorganizations",
                "listsecurityconfigurations",
                "listvaults"
            ],
            "LIGHTSAIL": [
                "getblueprints",
                "getbuckets",
                "getcertificates",
                "getcontainerservices",
                "getrelationaldatabases",
                "getstaticips"
            ],
            "S3": [
                "getbucketacl",
                "getbucketcors",
                "getbucketencryption",
                "getbucketintelligenttieringconfiguration",
                "getbucketlifecycle",
                "getbucketlocation",
                "getbucketlogging",
                "getbucketmetricsconfiguration",
                "getbucketnotification",
                "getbucketobjectlockconfiguration",
                "getbucketownershipcontrols",
                "getbucketreplication",
                "getbucketrequestpayment",
                "getbucketversioning",
                "getbucketwebsite",
                "listaccesspoints",
                "listbuckets",
                "listmultiregionaccesspoints",
                "liststoragelensgroups"
            ],
            "ATHENA": [
                "getdatacatalog",
                "getworkgroup",
                "listdatacatalogs",
                "listpreparedstatements",
                "listtagsforresource",
                "listworkgroups"
            ],
            "GUARDDUTY": [
                "getdetector",
                "getinvitationscount",
                "getmalwarescansettings",
                "getmasteraccount",
                "listdetectors",
                "listfilters",
                "listmembers",
                "listpublishingdestinations",
                "listthreatintelsets"
            ],
            "FRAUD": [
                "getdetectors",
                "getentitytypes",
                "geteventtypes",
                "getexternalmodels",
                "getoutcomes",
                "getvariables"
            ],
            "X_RAY": [
                "getencryptionconfig"
            ],
            "DATA": [
                "getlifecyclepolicies",
                "listdatasets"
            ],
            "MACIE": [
                "getmaciesession"
            ],
            "ROUTE": [
                "getresolverrule",
                "listfirewalldomainlists",
                "listresolverendpoints",
                "listresolverquerylogconfigs"
            ],
            "SERVICE": [
                "getservicequota",
                "listapplications",
                "listattributegroups"
            ],
            "SNS": [
                "gettopicattributes",
                "listsubscriptions",
                "listsubscriptionsbytopic",
                "listtagsforresource",
                "numberofmessagespublishedperaccount"
            ],
            "BEDROCK": [
                "listagents"
            ],
            "IOT": [
                "listalarmmodels",
                "listassetmodels",
                "listcacertificates",
                "listcertificates",
                "listcustommetrics",
                "listdetectormodels",
                "listdimensions",
                "listfleetmetrics",
                "listfuotatasks",
                "listgateways",
                "listinputs",
                "listmitigationactions",
                "listmulticastgroups",
                "listpolicies",
                "listportals",
                "listrolealiases",
                "listscheduledaudits",
                "listsecurityprofiles",
                "listserviceprofiles",
                "listthings",
                "listtopicruledestinations",
                "listworkspaces"
            ],
            "ACCESS": [
                "listanalyzers",
                "listarchiverules"
            ],
            "APPINTEGRATIONS": [
                "listapplications",
                "listeventintegrations"
            ],
            "MEDIACONNECT": [
                "listbridges",
                "listflows",
                "listgateways"
            ],
            "TRANSCRIBE": [
                "listcallanalyticsjobs",
                "listlanguagemodels",
                "listmedicalscribejobs",
                "listmedicaltranscriptionjobs",
                "listmedicalvocabularies",
                "listvocabularies"
            ],
            "PRIVATE": [
                "listcertificateauthorities"
            ],
            "CERTIFICATE": [
                "listcertificates"
            ],
            "TRANSFER": [
                "listcertificates",
                "listprofiles",
                "listservers",
                "listworkflows"
            ],
            "EKS": [
                "listclusters"
            ],
            "SAGEMAKER": [
                "listcoderepositories",
                "listdomains",
                "listendpointconfigs",
                "listendpoints",
                "listfeaturegroups",
                "listimages",
                "listinferenceexperiments",
                "listnotebookinstancelifecycleconfigs",
                "listpipelines",
                "listworkteams"
            ],
            "CLEAN": [
                "listcollaborations"
            ],
            "GREENGRASS": [
                "listcomponents",
                "listconnectordefinitions",
                "listdevicedefinitions",
                "listfunctiondefinitions",
                "listgroups",
                "listloggerdefinitions",
                "listresourcedefinitions"
            ],
            "IMAGE": [
                "listcomponents",
                "listcontainerrecipes",
                "listdistributionconfigurations",
                "listimagerecipes",
                "listimages",
                "listinfrastructureconfigurations"
            ],
            "GROUND": [
                "listconfigs",
                "listmissionprofiles"
            ],
            "MEDIASTORE": [
                "listcontainers"
            ],
            "DATABREW": [
                "listdatasets",
                "listprojects",
                "listrecipes",
                "listrulesets",
                "listschedules"
            ],
            "COMPREHEND": [
                "listdocumentclassifiers",
                "listdominantlanguagedetectionjobs",
                "listentitiesdetectionjobs",
                "listentityrecognizers",
                "listeventsdetectionjobs",
                "listflywheels",
                "listkeyphrasesdetectionjobs",
                "listpiientitiesdetectionjobs",
                "listsentimentdetectionjobs",
                "listtopicsdetectionjobs"
            ],
            "CUSTOMER": [
                "listdomains"
            ],
            "NETWORK": [
                "listfirewallpolicies",
                "listfirewalls"
            ],
            "APPFLOW": [
                "listflows"
            ],
            "NEPTUNEGRAPH": [
                "listgraphs",
                "listgraphsnapshots"
            ],
            "DETECTIVE": [
                "listgraphs"
            ],
            "CONNECT": [
                "listinstances",
                "listphonenumbersv2"
            ],
            "QLDB": [
                "listledgers"
            ],
            "IVSCHAT": [
                "listloggingconfigurations"
            ],
            "LOCATION": [
                "listmaps",
                "listtrackers"
            ],
            "CODESTAR": [
                "listnotificationrules"
            ],
            "TRANSLATE": [
                "listparalleldata",
                "listterminologies",
                "listtexttranslationjobs"
            ],
            "CODEPIPELINE": [
                "listpipelines"
            ],
            "MEDIATAILOR": [
                "listplaybackconfigurations"
            ],
            "IVS": [
                "listplaybackkeypairs",
                "listrecordingconfigurations"
            ],
            "MEDIACONVERT": [
                "listpresets"
            ],
            "LOOKOUT": [
                "listprojects"
            ],
            "CODEBUILD": [
                "listprojects",
                "listreportgroups"
            ],
            "CODECOMMIT": [
                "listrepositories"
            ],
            "ROBOMAKER": [
                "listrobotapplications",
                "listsimulationapplications"
            ],
            "SCHEDULER": [
                "listschedulegroups",
                "listschedules"
            ],
            "CLOUD": [
                "listservices"
            ],
            "OUTPOSTS": [
                "listsites"
            ],
            "ECS": [
                "listtagsforresource"
            ],
            "DATASYNC": [
                "listtasks"
            ],
            "COGNITO": [
                "listuserpools",
                "userpoolread"
            ],
            "GRAFANA": [
                "listworkspaces"
            ]
        }

        self._services = CloudWatchAWSServicesMap()

    def get(self, operation_name: CloudWatchAPIOperationName):
        return self._api_ops_map[operation_name]
    
    def get_service_operations(self, service_name: CloudWatchServiceName) -> list[CloudWatchAPIOperationName]:
        service = self._services.get(service_name)
        return self._service_api_ops_map.get(service.name, [])



from pydantic import BaseModel, StrictStr, StrictInt, StrictBool
from typing import Literal
from .elasticache_log_delivery_configuration import ElasticacheLogDeliveryConfiguration


class ElasticachePendingModifiedValues(BaseModel):
    NumCacheNodes: StrictInt
    CacheNodeIdsToRemove: list[StrictStr]
    EngineVersion: StrictStr
    CacheNodeType: StrictStr
    AuthTokenStatus: Literal['SETTING', 'ROTATING']
    LogDeliveryConfigruations: list[ElasticacheLogDeliveryConfiguration]
    TransitEncryptionEnabled: StrictBool
    TransitEncryptionMode: Literal['preferred', 'required']


import datetime
from pydantic import BaseModel, StrictStr, StrictBool
from typing import Literal


EC2AttachmentState = Literal[
    "attaching",
    "attached",
    "detaching",
    "detached",
    "busy",
]

class EC2Attachment(BaseModel):
    DeleteOnTermination: StrictBool = False
    AssociatedResource: StrictStr
    InstanceOwningService: StrictStr
    VolumeId: StrictStr
    InstanceId: StrictStr
    Device: StrictStr
    State: EC2AttachmentState
    AttachTime: datetime.datetime


from pydantic import BaseModel, StrictStr


class ElasticacheCacheParameterGroup(BaseModel):
    CacheParameterGroupName: StrictStr
    ParameterApplyStatus: StrictStr
    CacheNodeIdsToReboot: list[StrictStr]


from pydantic import BaseModel, StrictStr
from typing import Literal


GroupTypes = Literal[
    "DIMENSION",
    "TAG",
    "COST_CATEGORY"
]


class GroupDefinition(BaseModel):
    Type: GroupTypes
    Key: StrictStr


import datetime
from pydantic import BaseModel, StrictStr
from .elasticache_endpoint import ElasticacheEndpoint


class ElasticacheCacheNode(BaseModel):
    CacheNodeId: StrictStr
    CacheNodeStatus: StrictStr
    CacheNodeCreateTime: datetime.datetime
    Endpoint: ElasticacheEndpoint
    ParameterGroupStatus: StrictStr
    SourceCacheNodeId: StrictStr
    CustomerAvailabilityZone: StrictStr
    CustomerOutpostArn: StrictStr



from pydantic import BaseModel, StrictStr, StrictInt


class ElasticacheDescribeReplicationGroupsRequest(BaseModel):
        ReplicationGroupId: StrictStr = None
        MaxRecords: StrictInt = 100
        Marker: StrictStr | None = None
